@@ -1,6 +1,6 @@
 /* GStreamer
  * Copyright (C) 2008 Tristan Matthews <tristan@sat.qc.ca>
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -151,7 +151,7 @@ gst_jack_audio_src_free_channels (GstJackAudioSrc * src)
 static GType
 gst_jack_ring_buffer_get_type (void)
 {
-  static volatile gsize ringbuffer_type = 0;
+  static gsize ringbuffer_type = 0;
 
   if (g_once_init_enter (&ringbuffer_type)) {
     static const GTypeInfo ringbuffer_info = { sizeof (GstJackRingBufferClass),
@@ -407,7 +407,6 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
 {
   GstJackAudioSrc *src;
   GstJackRingBuffer *abuf;
-  const char **ports;
   gint sample_rate, buffer_size;
   gint i, bpf, rate, channels, res;
   jack_client_t *client;
@@ -443,7 +442,9 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
       (GST_SECOND / GST_USECOND), rate * bpf);
   /* segtotal based on buffer-time latency */
   spec->segtotal = spec->buffer_time / spec->latency_time;
-  if (spec->segtotal < 2) {
+
+  /* Use small period when low-latency is enabled regardless of buffer-time */
+  if (spec->segtotal < 2 || src->low_latency) {
     spec->segtotal = 2;
     spec->buffer_time = spec->latency_time * spec->segtotal;
   }
@@ -465,20 +466,38 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
   /* if we need to automatically connect the ports, do so now. We must do this
    * after activating the client. */
   if (src->connect == GST_JACK_CONNECT_AUTO
-      || src->connect == GST_JACK_CONNECT_AUTO_FORCED) {
+      || src->connect == GST_JACK_CONNECT_AUTO_FORCED
+      || src->connect == GST_JACK_CONNECT_EXPLICIT) {
+    const char **available_ports = NULL;
+    const char **jack_ports = NULL;
+    char **user_ports = NULL;
+
     /* find all the physical output ports. A physical output port is a port
      * associated with a hardware device. Someone needs connect to a physical
      * port in order to capture something. */
 
-    if (src->port_pattern == NULL) {
-      ports = jack_get_ports (client, NULL, NULL,
-          JackPortIsPhysical | JackPortIsOutput);
-    } else {
-      ports = jack_get_ports (client, src->port_pattern, NULL,
-          JackPortIsOutput);
+    if (src->port_names) {
+      user_ports = gst_jack_audio_client_get_port_names_from_string (client,
+          src->port_names, JackPortIsOutput);
+
+      if (user_ports)
+        available_ports = (const char **) user_ports;
     }
 
-    if (ports == NULL) {
+    if (!available_ports && src->connect == GST_JACK_CONNECT_EXPLICIT)
+      goto wrong_port_names;
+
+    if (!available_ports) {
+      if (!src->port_pattern) {
+        jack_ports = jack_get_ports (client, NULL, NULL,
+            JackPortIsPhysical | JackPortIsOutput);
+      } else {
+        jack_ports = jack_get_ports (client, src->port_pattern, NULL,
+            JackPortIsOutput);
+      }
+    }
+
+    if (!available_ports) {
       /* no ports? fine then we don't do anything except for posting a warning
        * message. */
       GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND, (NULL),
@@ -488,7 +507,7 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
 
     for (i = 0; i < channels; i++) {
       /* stop when all output ports are exhausted */
-      if (ports[i] == NULL) {
+      if (!available_ports[i]) {
         /* post a warning that we could not connect all ports */
         GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND, (NULL),
             ("No more physical ports, leaving some ports unconnected"));
@@ -498,11 +517,18 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
           jack_port_name (src->ports[i]));
 
       /* connect the physical port to a port */
-      res = jack_connect (client, ports[i], jack_port_name (src->ports[i]));
-      if (res != 0 && res != EEXIST)
+      res = jack_connect (client,
+          available_ports[i], jack_port_name (src->ports[i]));
+      if (res != 0 && res != EEXIST) {
+        jack_free (jack_ports);
+        g_strfreev (user_ports);
+
         goto cannot_connect;
+      }
     }
-    jack_free (ports);
+
+    jack_free (jack_ports);
+    g_strfreev (user_ports);
   }
 done:
 
@@ -537,7 +563,12 @@ cannot_connect:
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("Could not connect input ports to physical ports (%d:%s)",
             res, g_strerror (res)));
-    jack_free (ports);
+    return FALSE;
+  }
+wrong_port_names:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
+        ("Invalid port-names was provided"));
     return FALSE;
   }
 }
@@ -687,6 +718,8 @@ enum
 #define DEFAULT_PROP_CLIENT_NAME	NULL
 #define DEFAULT_PROP_TRANSPORT	GST_JACK_TRANSPORT_AUTONOMOUS
 #define DEFAULT_PROP_PORT_PATTERN     	NULL
+#define DEFAULT_PROP_LOW_LATENCY  FALSE
+
 enum
 {
   PROP_0,
@@ -696,6 +729,8 @@ enum
   PROP_CLIENT_NAME,
   PROP_PORT_PATTERN,
   PROP_TRANSPORT,
+  PROP_LOW_LATENCY,
+  PROP_PORT_NAMES,
   PROP_LAST
 };
 
@@ -715,6 +750,8 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_jack_audio_src_parent_class parent_class
 G_DEFINE_TYPE (GstJackAudioSrc, gst_jack_audio_src, GST_TYPE_AUDIO_BASE_SRC);
+GST_ELEMENT_REGISTER_DEFINE (jackaudiosrc, "jackaudiosrc",
+    GST_RANK_PRIMARY, GST_TYPE_JACK_AUDIO_SRC);
 
 static void gst_jack_audio_src_dispose (GObject * object);
 static void gst_jack_audio_src_set_property (GObject * object, guint prop_id,
@@ -801,6 +838,36 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           GST_TYPE_JACK_TRANSPORT, DEFAULT_PROP_TRANSPORT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstJackAudioSrc:low-latency:
+   *
+   * Optimize all settings for lowest latency. When enabled,
+   * #GstAudioBaseSrc:buffer-time and #GstAudioBaseSrc:latency-time will be
+   * ignored.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_LOW_LATENCY,
+      g_param_spec_boolean ("low-latency", "Low latency",
+          "Optimize all settings for lowest latency. When enabled, "
+          "\"buffer-time\" and \"latency-time\" will be ignored",
+          DEFAULT_PROP_LOW_LATENCY,
+          GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
+          G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstJackAudioSrc:port-names:
+   *
+   * Comma-separated list of port name including "client_name:" prefix
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_PORT_NAMES,
+      g_param_spec_string ("port-names", "Port Names",
+          "Comma-separated list of port name including \"client_name:\" prefix",
+          NULL, GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
+          G_PARAM_STATIC_STRINGS));
+
   gst_element_class_add_static_pad_template (gstelement_class, &src_factory);
 
   gst_element_class_set_static_metadata (gstelement_class,
@@ -831,6 +898,7 @@ gst_jack_audio_src_init (GstJackAudioSrc * src)
   src->buffers = NULL;
   src->client_name = g_strdup (DEFAULT_PROP_CLIENT_NAME);
   src->transport = DEFAULT_PROP_TRANSPORT;
+  src->low_latency = DEFAULT_PROP_LOW_LATENCY;
 }
 
 static void
@@ -850,6 +918,8 @@ gst_jack_audio_src_dispose (GObject * object)
     src->port_pattern = NULL;
   }
 
+  g_clear_pointer (&src->port_names, g_free);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -884,6 +954,13 @@ gst_jack_audio_src_set_property (GObject * object, guint prop_id,
     case PROP_TRANSPORT:
       src->transport = g_value_get_flags (value);
       break;
+    case PROP_LOW_LATENCY:
+      src->low_latency = g_value_get_boolean (value);
+      break;
+    case PROP_PORT_NAMES:
+      g_free (src->port_names);
+      src->port_names = g_value_dup_string (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -915,6 +992,12 @@ gst_jack_audio_src_get_property (GObject * object, guint prop_id,
     case PROP_TRANSPORT:
       g_value_set_flags (value, src->transport);
       break;
+    case PROP_LOW_LATENCY:
+      g_value_set_boolean (value, src->low_latency);
+      break;
+    case PROP_PORT_NAMES:
+      g_value_set_string (value, src->port_names);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -933,14 +1016,42 @@ gst_jack_audio_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
   if (src->client == NULL)
     goto no_client;
 
+  if (src->connect == GST_JACK_CONNECT_EXPLICIT && !src->port_names)
+    goto no_port_names;
+
   client = gst_jack_audio_client_get_client (src->client);
 
-  if (src->connect == GST_JACK_CONNECT_AUTO) {
+  if (src->connect == GST_JACK_CONNECT_AUTO ||
+      src->connect == GST_JACK_CONNECT_EXPLICIT) {
+    max = 0;
+
+    if (src->port_names) {
+      gchar **user_ports =
+          gst_jack_audio_client_get_port_names_from_string (client,
+          src->port_names, JackPortIsOutput);
+
+      if (user_ports) {
+        max = g_strv_length (user_ports);
+      } else {
+        GST_ELEMENT_WARNING (src, RESOURCE, NOT_FOUND,
+            ("Invalid \"port-names\" was requested"),
+            ("Requested \"port-names\" %s contains invalid name",
+                src->port_names));
+      }
+
+      g_strfreev (user_ports);
+    }
+
+    if (max > 0)
+      goto found;
+
+    if (src->connect == GST_JACK_CONNECT_EXPLICIT)
+      goto no_port_names;
+
     /* get a port count, this is the number of channels we can automatically
      * connect. */
     ports = jack_get_ports (client, NULL, NULL,
         JackPortIsPhysical | JackPortIsOutput);
-    max = 0;
     if (ports != NULL) {
       for (; ports[max]; max++);
 
@@ -952,7 +1063,13 @@ gst_jack_audio_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
      * pads. */
     max = G_MAXINT;
   }
-  min = MIN (1, max);
+
+found:
+  if (src->connect == GST_JACK_CONNECT_EXPLICIT) {
+    min = max;
+  } else {
+    min = MIN (1, max);
+  }
 
   rate = jack_get_sample_rate (client);
 
@@ -961,9 +1078,13 @@ gst_jack_audio_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
   if (!src->caps) {
     src->caps = gst_caps_new_simple ("audio/x-raw",
         "format", G_TYPE_STRING, GST_JACK_FORMAT_STR,
-        "layout", G_TYPE_STRING, "interleaved",
-        "rate", G_TYPE_INT, rate,
-        "channels", GST_TYPE_INT_RANGE, min, max, NULL);
+        "layout", G_TYPE_STRING, "interleaved", "rate", G_TYPE_INT, rate, NULL);
+    if (min == max) {
+      gst_caps_set_simple (src->caps, "channels", G_TYPE_INT, min, NULL);
+    } else {
+      gst_caps_set_simple (src->caps,
+          "channels", GST_TYPE_INT_RANGE, min, max, NULL);
+    }
   }
   GST_INFO_OBJECT (src, "returning caps %" GST_PTR_FORMAT, src->caps);
 
@@ -976,6 +1097,13 @@ no_client:
     /* base class will get template caps for us when we return NULL */
     return NULL;
   }
+no_port_names:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS,
+        ("User must provide valid port names"),
+        ("\"port-names\" contains invalid name or NULL string"));
+    return NULL;
+  }
 }
 
 static GstAudioRingBuffer *
