@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) 2011 David Schleef <ds@entropywave.com>
+ * Copyright (C) 2021 Igalia S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -40,6 +41,9 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstbasesink.h>
+#include <gio/gio.h>
+
+#include "gstsoupelements.h"
 #include "gstsouphttpclientsink.h"
 #include "gstsouputils.h"
 
@@ -58,26 +62,23 @@ static void gst_soup_http_client_sink_finalize (GObject * object);
 
 static gboolean gst_soup_http_client_sink_set_caps (GstBaseSink * sink,
     GstCaps * caps);
-static void gst_soup_http_client_sink_get_times (GstBaseSink * sink,
-    GstBuffer * buffer, GstClockTime * start, GstClockTime * end);
 static gboolean gst_soup_http_client_sink_start (GstBaseSink * sink);
 static gboolean gst_soup_http_client_sink_stop (GstBaseSink * sink);
 static gboolean gst_soup_http_client_sink_unlock (GstBaseSink * sink);
-static gboolean gst_soup_http_client_sink_event (GstBaseSink * sink,
-    GstEvent * event);
-static GstFlowReturn gst_soup_http_client_sink_preroll (GstBaseSink * sink,
-    GstBuffer * buffer);
 static GstFlowReturn gst_soup_http_client_sink_render (GstBaseSink * sink,
     GstBuffer * buffer);
-
 static void gst_soup_http_client_sink_reset (GstSoupHttpClientSink *
     souphttpsink);
-static void authenticate (SoupSession * session, SoupMessage * msg,
-    SoupAuth * auth, gboolean retrying, gpointer user_data);
-static void callback (SoupSession * session, SoupMessage * msg,
-    gpointer user_data);
-static gboolean gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink *
-    souphttpsink, const gchar * uri);
+
+static gboolean authenticate (SoupMessage * msg, SoupAuth * auth,
+    gboolean retrying, gpointer user_data);
+static void restarted (SoupMessage * msg, GBytes * body);
+static gboolean send_handle_status (SoupMessage * msg, GError * error,
+    GstSoupHttpClientSink * sink);
+
+static gboolean
+gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink * souphttpsink,
+    const gchar * uri);
 
 enum
 {
@@ -114,6 +115,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 #define gst_soup_http_client_sink_parent_class parent_class
 G_DEFINE_TYPE (GstSoupHttpClientSink, gst_soup_http_client_sink,
     GST_TYPE_BASE_SINK);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (souphttpclientsink, "souphttpclientsink",
+    GST_RANK_NONE, GST_TYPE_SOUP_HTTP_CLIENT_SINK, soup_element_init (plugin));
 
 static void
 gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
@@ -166,7 +169,8 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_SESSION,
       g_param_spec_object ("session", "session",
           "SoupSession object to use for communication",
-          SOUP_TYPE_SESSION, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          _soup_session_get_type (),
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_COOKIES,
       g_param_spec_boxed ("cookies", "Cookies", "HTTP request cookies",
           G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
@@ -189,8 +193,8 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_SOUP_LOG_LEVEL,
       g_param_spec_enum ("http-log-level", "HTTP log level",
           "Set log level for soup's HTTP session log",
-          SOUP_TYPE_LOGGER_LOG_LEVEL, DEFAULT_SOUP_LOG_LEVEL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          _soup_logger_log_level_get_type (),
+          DEFAULT_SOUP_LOG_LEVEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_add_static_pad_template (gstelement_class,
       &gst_soup_http_client_sink_sink_template);
@@ -201,23 +205,15 @@ gst_soup_http_client_sink_class_init (GstSoupHttpClientSinkClass * klass)
 
   base_sink_class->set_caps =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_set_caps);
-  if (0)
-    base_sink_class->get_times =
-        GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_get_times);
   base_sink_class->start = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_start);
   base_sink_class->stop = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_stop);
   base_sink_class->unlock =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_unlock);
-  base_sink_class->event = GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_event);
-  if (0)
-    base_sink_class->preroll =
-        GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_preroll);
   base_sink_class->render =
       GST_DEBUG_FUNCPTR (gst_soup_http_client_sink_render);
 
   GST_DEBUG_CATEGORY_INIT (souphttpclientsink_dbg, "souphttpclientsink", 0,
       "souphttpclientsink element");
-
 }
 
 static void
@@ -240,6 +236,7 @@ gst_soup_http_client_sink_init (GstSoupHttpClientSink * souphttpsink)
   souphttpsink->log_level = DEFAULT_SOUP_LOG_LEVEL;
   souphttpsink->retry_delay = 5;
   souphttpsink->retries = 0;
+  souphttpsink->sent_buffers = NULL;
   proxy = g_getenv ("http_proxy");
   if (proxy && !gst_soup_http_client_sink_set_proxy (souphttpsink, proxy)) {
     GST_WARNING_OBJECT (souphttpsink,
@@ -275,15 +272,15 @@ gst_soup_http_client_sink_set_proxy (GstSoupHttpClientSink * souphttpsink,
     const gchar * uri)
 {
   if (souphttpsink->proxy) {
-    soup_uri_free (souphttpsink->proxy);
+    gst_soup_uri_free (souphttpsink->proxy);
     souphttpsink->proxy = NULL;
   }
   if (g_str_has_prefix (uri, "http://")) {
-    souphttpsink->proxy = soup_uri_new (uri);
+    souphttpsink->proxy = gst_soup_uri_new (uri);
   } else {
     gchar *new_uri = g_strconcat ("http://", uri, NULL);
 
-    souphttpsink->proxy = soup_uri_new (new_uri);
+    souphttpsink->proxy = gst_soup_uri_new (new_uri);
     g_free (new_uri);
   }
 
@@ -412,7 +409,7 @@ gst_soup_http_client_sink_get_property (GObject * object, guint property_id,
       if (souphttpsink->proxy == NULL)
         g_value_set_static_string (value, "");
       else {
-        char *proxy = soup_uri_to_string (souphttpsink->proxy, FALSE);
+        char *proxy = gst_soup_uri_to_string (souphttpsink->proxy);
 
         g_value_set_string (value, proxy);
         g_free (proxy);
@@ -462,7 +459,7 @@ gst_soup_http_client_sink_finalize (GObject * object)
   g_free (souphttpsink->proxy_id);
   g_free (souphttpsink->proxy_pw);
   if (souphttpsink->proxy)
-    soup_uri_free (souphttpsink->proxy);
+    gst_soup_uri_free (souphttpsink->proxy);
   g_free (souphttpsink->location);
   g_strfreev (souphttpsink->cookies);
 
@@ -472,8 +469,6 @@ gst_soup_http_client_sink_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
-
 static gboolean
 gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
 {
@@ -505,13 +500,6 @@ gst_soup_http_client_sink_set_caps (GstBaseSink * sink, GstCaps * caps)
   return TRUE;
 }
 
-static void
-gst_soup_http_client_sink_get_times (GstBaseSink * sink, GstBuffer * buffer,
-    GstClockTime * start, GstClockTime * end)
-{
-
-}
-
 static gboolean
 thread_ready_idle_cb (gpointer data)
 {
@@ -530,11 +518,38 @@ static gpointer
 thread_func (gpointer ptr)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (ptr);
+  GProxyResolver *proxy_resolver;
+  GMainContext *context;
 
   GST_DEBUG ("thread start");
 
+  context = souphttpsink->context;
+  g_main_context_push_thread_default (context);
+
+  if (souphttpsink->proxy != NULL) {
+    char *proxy_string = gst_soup_uri_to_string (souphttpsink->proxy);
+    proxy_resolver = g_simple_proxy_resolver_new (proxy_string, NULL);
+    g_free (proxy_string);
+  } else
+    proxy_resolver = g_object_ref (g_proxy_resolver_get_default ());
+
+  souphttpsink->session =
+      _soup_session_new_with_options ("user-agent", souphttpsink->user_agent,
+      "timeout", souphttpsink->timeout, "proxy-resolver", proxy_resolver, NULL);
+
+  g_object_unref (proxy_resolver);
+
+  if (gst_soup_loader_get_api_version () < 3) {
+    g_signal_connect (souphttpsink->session, "authenticate",
+        G_CALLBACK (authenticate), souphttpsink);
+  }
+
+  GST_DEBUG ("created session");
+
   g_main_loop_run (souphttpsink->loop);
 
+  g_main_context_pop_thread_default (context);
+
   GST_DEBUG ("thread quit");
 
   return NULL;
@@ -560,7 +575,7 @@ gst_soup_http_client_sink_start (GstBaseSink * sink)
     g_source_attach (source, souphttpsink->context);
     g_source_unref (source);
 
-    souphttpsink->loop = g_main_loop_new (souphttpsink->context, TRUE);
+    souphttpsink->loop = g_main_loop_new (souphttpsink->context, FALSE);
 
     g_mutex_lock (&souphttpsink->mutex);
 
@@ -576,32 +591,15 @@ gst_soup_http_client_sink_start (GstBaseSink * sink)
     }
 
     GST_LOG_OBJECT (souphttpsink, "waiting for main loop thread to start up");
-    g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
+    while (!g_main_loop_is_running (souphttpsink->loop))
+      g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
     g_mutex_unlock (&souphttpsink->mutex);
     GST_LOG_OBJECT (souphttpsink, "main loop thread running");
-
-    if (souphttpsink->proxy == NULL) {
-      souphttpsink->session =
-          soup_session_async_new_with_options (SOUP_SESSION_ASYNC_CONTEXT,
-          souphttpsink->context, SOUP_SESSION_USER_AGENT,
-          souphttpsink->user_agent, SOUP_SESSION_TIMEOUT, souphttpsink->timeout,
-          SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_PROXY_RESOLVER_DEFAULT,
-          NULL);
-    } else {
-      souphttpsink->session =
-          soup_session_async_new_with_options (SOUP_SESSION_ASYNC_CONTEXT,
-          souphttpsink->context, SOUP_SESSION_USER_AGENT,
-          souphttpsink->user_agent, SOUP_SESSION_TIMEOUT, souphttpsink->timeout,
-          SOUP_SESSION_PROXY_URI, souphttpsink->proxy, NULL);
-    }
-
-    g_signal_connect (souphttpsink->session, "authenticate",
-        G_CALLBACK (authenticate), souphttpsink);
   }
 
   /* Set up logging */
   gst_soup_util_log_setup (souphttpsink->session, souphttpsink->log_level,
-      GST_ELEMENT (souphttpsink));
+      G_OBJECT (souphttpsink));
 
   return TRUE;
 }
@@ -614,7 +612,7 @@ gst_soup_http_client_sink_stop (GstBaseSink * sink)
   GST_DEBUG ("stop");
 
   if (souphttpsink->prop_session == NULL) {
-    soup_session_abort (souphttpsink->session);
+    _soup_session_abort (souphttpsink->session);
     g_object_unref (souphttpsink->session);
   }
 
@@ -653,40 +651,13 @@ gst_soup_http_client_sink_unlock (GstBaseSink * sink)
   return TRUE;
 }
 
-static gboolean
-gst_soup_http_client_sink_event (GstBaseSink * sink, GstEvent * event)
-{
-  GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (sink);
-
-  GST_DEBUG_OBJECT (souphttpsink, "event");
-
-  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
-    GST_DEBUG_OBJECT (souphttpsink, "got eos");
-    g_mutex_lock (&souphttpsink->mutex);
-    while (souphttpsink->message) {
-      GST_DEBUG_OBJECT (souphttpsink, "waiting");
-      g_cond_wait (&souphttpsink->cond, &souphttpsink->mutex);
-    }
-    g_mutex_unlock (&souphttpsink->mutex);
-    GST_DEBUG_OBJECT (souphttpsink, "finished eos");
-  }
-
-  return GST_BASE_SINK_CLASS (parent_class)->event (sink, event);
-}
-
-static GstFlowReturn
-gst_soup_http_client_sink_preroll (GstBaseSink * sink, GstBuffer * buffer)
-{
-  GST_DEBUG ("preroll");
-
-  return GST_FLOW_OK;
-}
-
 static void
 send_message_locked (GstSoupHttpClientSink * souphttpsink)
 {
   GList *g;
   guint64 n;
+  GByteArray *array;
+  GInputStream *in_stream;
 
   if (souphttpsink->queued_buffers == NULL || souphttpsink->message) {
     return;
@@ -701,7 +672,7 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
     return;
   }
 
-  souphttpsink->message = soup_message_new ("PUT", souphttpsink->location);
+  souphttpsink->message = _soup_message_new ("PUT", souphttpsink->location);
   if (souphttpsink->message == NULL) {
     GST_WARNING_OBJECT (souphttpsink,
         "URI could not be parsed while creating message.");
@@ -711,18 +682,21 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
     return;
   }
 
-  soup_message_set_flags (souphttpsink->message,
+  g_signal_connect (souphttpsink->message, "restarted", G_CALLBACK (restarted),
+      souphttpsink->request_body);
+
+  _soup_message_set_flags (souphttpsink->message,
       (souphttpsink->automatic_redirect ? 0 : SOUP_MESSAGE_NO_REDIRECT));
 
   if (souphttpsink->cookies) {
     gchar **cookie;
 
     for (cookie = souphttpsink->cookies; *cookie != NULL; cookie++) {
-      soup_message_headers_append (souphttpsink->message->request_headers,
-          "Cookie", *cookie);
+      _soup_message_headers_append (_soup_message_get_request_headers
+          (souphttpsink->message), "Cookie", *cookie);
     }
   }
-
+  array = g_byte_array_new ();
   n = 0;
   if (souphttpsink->offset == 0) {
     for (g = souphttpsink->streamheader_buffers; g; g = g_list_next (g)) {
@@ -731,11 +705,7 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
 
       GST_DEBUG_OBJECT (souphttpsink, "queueing stream headers");
       gst_buffer_map (buffer, &map, GST_MAP_READ);
-      /* Stream headers are updated whenever ::set_caps is called, so there's
-       * no guarantees about their lifetime and we ask libsoup to copy them 
-       * into the message body with SOUP_MEMORY_COPY. */
-      soup_message_body_append (souphttpsink->message->request_body,
-          SOUP_MEMORY_COPY, map.data, map.size);
+      g_byte_array_append (array, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
@@ -747,23 +717,24 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
       GstMapInfo map;
 
       gst_buffer_map (buffer, &map, GST_MAP_READ);
-      /* Queued buffers are only freed in the next iteration of the mainloop
-       * after the message body has been written out, so we don't need libsoup
-       * to copy those while appending to the body. However, if the buffer is
-       * used elsewhere, it should be copied. Hence, SOUP_MEMORY_TEMPORARY. */
-      soup_message_body_append (souphttpsink->message->request_body,
-          SOUP_MEMORY_TEMPORARY, map.data, map.size);
+      g_byte_array_append (array, map.data, map.size);
       n += map.size;
       gst_buffer_unmap (buffer, &map);
     }
   }
 
+  {
+    souphttpsink->request_body = g_byte_array_free_to_bytes (array);
+    _soup_message_set_request_body_from_bytes (souphttpsink->message,
+        NULL, souphttpsink->request_body);
+  }
+
   if (souphttpsink->offset != 0) {
     char *s;
     s = g_strdup_printf ("bytes %" G_GUINT64_FORMAT "-%" G_GUINT64_FORMAT "/*",
         souphttpsink->offset, souphttpsink->offset + n - 1);
-    soup_message_headers_append (souphttpsink->message->request_headers,
-        "Content-Range", s);
+    _soup_message_headers_append (_soup_message_get_request_headers
+        (souphttpsink->message), "Content-Range", s);
     g_free (s);
   }
 
@@ -773,20 +744,35 @@ send_message_locked (GstSoupHttpClientSink * souphttpsink)
     g_list_free_full (souphttpsink->queued_buffers,
         (GDestroyNotify) gst_buffer_unref);
     souphttpsink->queued_buffers = NULL;
-    g_object_unref (souphttpsink->message);
-    souphttpsink->message = NULL;
+    g_clear_object (&souphttpsink->message);
+    g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
     return;
   }
 
+  in_stream =
+      _soup_session_send (souphttpsink->session, souphttpsink->message, NULL,
+      NULL);
+  if (in_stream == NULL) {
+    GError *error = NULL;
+
+    if (!send_handle_status (souphttpsink->message, error, souphttpsink)) {
+      g_object_unref (souphttpsink->message);
+      g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
+      g_clear_error (&error);
+      return;
+    }
+  }
   souphttpsink->sent_buffers = souphttpsink->queued_buffers;
+
+  g_clear_pointer (&souphttpsink->request_body, g_bytes_unref);
+  g_object_unref (in_stream);
+
+  g_list_free_full (souphttpsink->sent_buffers,
+      (GDestroyNotify) gst_buffer_unref);
+  souphttpsink->sent_buffers = NULL;
+  souphttpsink->failures = 0;
   souphttpsink->queued_buffers = NULL;
-
-  GST_DEBUG_OBJECT (souphttpsink,
-      "queue message %" G_GUINT64_FORMAT " %" G_GUINT64_FORMAT,
-      souphttpsink->offset, n);
-  soup_session_queue_message (souphttpsink->session, souphttpsink->message,
-      callback, souphttpsink);
-
+  g_clear_object (&souphttpsink->message);
   souphttpsink->offset += n;
 }
 
@@ -805,64 +791,67 @@ send_message (GstSoupHttpClientSink * souphttpsink)
   return FALSE;
 }
 
-static void
-callback (SoupSession * session, SoupMessage * msg, gpointer user_data)
+static gboolean
+send_handle_status (SoupMessage * msg, GError * error,
+    GstSoupHttpClientSink * sink)
 {
-  GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (user_data);
+  if (error) {
+    GST_DEBUG_OBJECT (sink, "callback error=%d %s",
+        error->code, error->message);
+  } else {
+    GST_DEBUG_OBJECT (sink, "callback status=%d %s",
+        _soup_message_get_status (msg), _soup_message_get_reason_phrase (msg));
+  }
 
-  GST_DEBUG_OBJECT (souphttpsink, "callback status=%d %s",
-      msg->status_code, msg->reason_phrase);
-
-  g_mutex_lock (&souphttpsink->mutex);
-  g_cond_signal (&souphttpsink->cond);
-  souphttpsink->message = NULL;
-
-  if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-    souphttpsink->failures++;
-    if (souphttpsink->retries &&
-        (souphttpsink->retries < 0 ||
-            souphttpsink->retries >= souphttpsink->failures)) {
+  if (error || !SOUP_STATUS_IS_SUCCESSFUL (_soup_message_get_status (msg))) {
+    sink->failures++;
+    if (sink->retries && (sink->retries < 0 || sink->retries >= sink->failures)) {
       guint64 retry_delay;
-      const char *retry_after =
-          soup_message_headers_get_one (msg->response_headers,
-          "Retry-After");
+      const char *retry_after;
+      SoupMessageHeaders *res_hdrs;
+      if (error) {
+        retry_delay = sink->retry_delay;
+        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
+            "error: %d %s (retrying PUT after %" G_GINT64_FORMAT
+            " seconds)", error->code, error->message, retry_delay);
+        goto err_done;
+      }
+      res_hdrs = _soup_message_get_response_headers (msg);
+      retry_after = _soup_message_headers_get_one (res_hdrs, "Retry-After");
       if (retry_after) {
         gchar *end = NULL;
         retry_delay = g_ascii_strtoull (retry_after, &end, 10);
         if (end || errno) {
-          retry_delay = souphttpsink->retry_delay;
+          retry_delay = sink->retry_delay;
         } else {
-          retry_delay = MAX (retry_delay, souphttpsink->retry_delay);
+          retry_delay = MAX (retry_delay, sink->retry_delay);
         }
-        GST_WARNING_OBJECT (souphttpsink, "Could not write to HTTP URI: "
+        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
             "status: %d %s (retrying PUT after %" G_GINT64_FORMAT
-            " seconds with Retry-After: %s)", msg->status_code,
-            msg->reason_phrase, retry_delay, retry_after);
+            " seconds with Retry-After: %s)",
+            _soup_message_get_status (msg),
+            _soup_message_get_reason_phrase (msg), retry_delay, retry_after);
       } else {
-        retry_delay = souphttpsink->retry_delay;
-        GST_WARNING_OBJECT (souphttpsink, "Could not write to HTTP URI: "
+        retry_delay = sink->retry_delay;
+        GST_WARNING_OBJECT (sink, "Could not write to HTTP URI: "
             "status: %d %s (retrying PUT after %" G_GINT64_FORMAT
-            " seconds)", msg->status_code, msg->reason_phrase, retry_delay);
+            " seconds)",
+            _soup_message_get_status (msg),
+            _soup_message_get_reason_phrase (msg), retry_delay);
       }
-      souphttpsink->timer = g_timeout_source_new_seconds (retry_delay);
-      g_source_set_callback (souphttpsink->timer, (GSourceFunc) (send_message),
-          souphttpsink, NULL);
-      g_source_attach (souphttpsink->timer, souphttpsink->context);
+    err_done:
+      sink->timer = g_timeout_source_new_seconds (retry_delay);
+      g_source_set_callback (sink->timer, (GSourceFunc) (send_message),
+          sink, NULL);
+      g_source_attach (sink->timer, sink->context);
     } else {
-      souphttpsink->status_code = msg->status_code;
-      souphttpsink->reason_phrase = g_strdup (msg->reason_phrase);
+      sink->status_code = _soup_message_get_status (msg);
+      sink->reason_phrase = g_strdup (_soup_message_get_reason_phrase (msg));
     }
-    g_mutex_unlock (&souphttpsink->mutex);
-    return;
+    return FALSE;
   }
 
-  g_list_free_full (souphttpsink->sent_buffers,
-      (GDestroyNotify) gst_buffer_unref);
-  souphttpsink->sent_buffers = NULL;
-  souphttpsink->failures = 0;
-
-  send_message_locked (souphttpsink);
-  g_mutex_unlock (&souphttpsink->mutex);
+  return TRUE;
 }
 
 static GstFlowReturn
@@ -870,7 +859,6 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (sink);
   GSource *source;
-  gboolean wake;
 
   if (souphttpsink->status_code != 0) {
     GST_ELEMENT_ERROR (souphttpsink, RESOURCE, WRITE,
@@ -882,18 +870,15 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 
   g_mutex_lock (&souphttpsink->mutex);
   if (souphttpsink->location != NULL) {
-    wake = (souphttpsink->queued_buffers == NULL);
     souphttpsink->queued_buffers =
         g_list_append (souphttpsink->queued_buffers, gst_buffer_ref (buffer));
 
-    if (wake) {
-      GST_DEBUG_OBJECT (souphttpsink, "setting callback for new buffers");
-      source = g_idle_source_new ();
-      g_source_set_callback (source, (GSourceFunc) (send_message),
-          souphttpsink, NULL);
-      g_source_attach (source, souphttpsink->context);
-      g_source_unref (source);
-    }
+    GST_DEBUG_OBJECT (souphttpsink, "setting callback for new buffers");
+    source = g_idle_source_new ();
+    g_source_set_callback (source, (GSourceFunc) (send_message),
+        souphttpsink, NULL);
+    g_source_attach (source, souphttpsink->context);
+    g_source_unref (source);
   }
   g_mutex_unlock (&souphttpsink->mutex);
 
@@ -901,21 +886,29 @@ gst_soup_http_client_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 }
 
 static void
-authenticate (SoupSession * session, SoupMessage * msg,
-    SoupAuth * auth, gboolean retrying, gpointer user_data)
+restarted (SoupMessage * msg, GBytes * body)
+{
+  _soup_message_set_request_body_from_bytes (msg, NULL, body);
+}
+
+static gboolean
+authenticate (SoupMessage * msg, SoupAuth * auth,
+    gboolean retrying, gpointer user_data)
 {
   GstSoupHttpClientSink *souphttpsink = GST_SOUP_HTTP_CLIENT_SINK (user_data);
 
   if (!retrying) {
+    SoupStatus status_code = _soup_message_get_status (msg);
     /* First time authentication only, if we fail and are called again with retry true fall through */
-    if (msg->status_code == SOUP_STATUS_UNAUTHORIZED) {
+    if (status_code == SOUP_STATUS_UNAUTHORIZED) {
       if (souphttpsink->user_id && souphttpsink->user_pw)
-        soup_auth_authenticate (auth, souphttpsink->user_id,
+        _soup_auth_authenticate (auth, souphttpsink->user_id,
             souphttpsink->user_pw);
-    } else if (msg->status_code == SOUP_STATUS_PROXY_AUTHENTICATION_REQUIRED) {
+    } else if (status_code == SOUP_STATUS_PROXY_AUTHENTICATION_REQUIRED) {
       if (souphttpsink->proxy_id && souphttpsink->proxy_pw)
-        soup_auth_authenticate (auth, souphttpsink->proxy_id,
+        _soup_auth_authenticate (auth, souphttpsink->proxy_id,
             souphttpsink->proxy_pw);
     }
   }
+  return FALSE;
 }
