@@ -21,20 +21,19 @@
  */
 /**
  * SECTION:element-vp8dec
+ * @title: vp8dec
  * @see_also: vp8enc, matroskademux
  *
  * This element decodes VP8 streams into raw video.
- * <ulink url="http://www.webmproject.org">VP8</ulink> is a royalty-free
- * video codec maintained by <ulink url="http://www.google.com/">Google
- * </ulink>. It's the successor of On2 VP3, which was the base of the
- * Theora video codec.
+ * [VP8](http://www.webmproject.org) is a royalty-free video codec maintained by
+ * [Google](http://www.google.com/). It's the successor of On2 VP3, which was
+ * the base of the Theora video codec.
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 -v filesrc location=videotestsrc.webm ! matroskademux ! vp8dec ! videoconvert ! videoscale ! autovideosink
  * ]| This example pipeline will decode a WebM stream and decodes the VP8 video.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +44,7 @@
 
 #include <string.h>
 
+#include "gstvpxelements.h"
 #include "gstvp8dec.h"
 #include "gstvp8utils.h"
 
@@ -60,6 +60,7 @@ static void gst_vp8_dec_set_default_format (GstVPXDec * dec, GstVideoFormat fmt,
     int width, int height);
 static void gst_vp8_dec_handle_resolution_change (GstVPXDec * dec,
     vpx_image_t * img, GstVideoFormat fmt);
+static gboolean gst_vp8_dec_get_needs_sync_point (GstVPXDec * dec);
 
 static GstStaticPadTemplate gst_vp8_dec_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -77,6 +78,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
 
 #define parent_class gst_vp8_dec_parent_class
 G_DEFINE_TYPE (GstVP8Dec, gst_vp8_dec, GST_TYPE_VPX_DEC);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (vp8dec, "vp8dec", GST_RANK_PRIMARY,
+    gst_vp8_dec_get_type (), vpx_element_init (plugin));
 
 static void
 gst_vp8_dec_class_init (GstVP8DecClass * klass)
@@ -87,11 +90,10 @@ gst_vp8_dec_class_init (GstVP8DecClass * klass)
   element_class = GST_ELEMENT_CLASS (klass);
   vpx_class = GST_VPX_DEC_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vp8_dec_sink_template));
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vp8_dec_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vp8_dec_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vp8_dec_src_template);
 
   gst_element_class_set_static_metadata (element_class,
       "On2 VP8 Decoder",
@@ -105,6 +107,8 @@ gst_vp8_dec_class_init (GstVP8DecClass * klass)
       GST_DEBUG_FUNCPTR (gst_vp8_dec_set_default_format);
   vpx_class->handle_resolution_change =
       GST_DEBUG_FUNCPTR (gst_vp8_dec_handle_resolution_change);
+  vpx_class->get_needs_sync_point =
+      GST_DEBUG_FUNCPTR (gst_vp8_dec_get_needs_sync_point);
 
   GST_DEBUG_CATEGORY_INIT (gst_vp8dec_debug, "vp8dec", 0, "VP8 Decoder");
 }
@@ -155,4 +159,10 @@ gst_vp8_dec_handle_resolution_change (GstVPXDec * dec, vpx_image_t * img,
   }
 }
 
+static gboolean
+gst_vp8_dec_get_needs_sync_point (GstVPXDec * dec)
+{
+  return FALSE;
+}
+
 #endif /* HAVE_VP8_DECODER */
