@@ -21,20 +21,19 @@
  */
 /**
  * SECTION:element-vp9dec
+ * @title: vp9dec
  * @see_also: vp9enc, matroskademux
  *
  * This element decodes VP9 streams into raw video.
- * <ulink url="http://www.webmproject.org">VP9</ulink> is a royalty-free
- * video codec maintained by <ulink url="http://www.google.com/">Google
- * </ulink>. It's the successor of On2 VP3, which was the base of the
- * Theora video codec.
+ * [VP9](http://www.webmproject.org) is a royalty-free video codec maintained by
+ * [Google](http://www.google.com/) It's the successor of On2 VP3, which was the
+ * base of the Theora video codec.
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 -v filesrc location=videotestsrc.webm ! matroskademux ! vp9dec ! videoconvert ! videoscale ! autovideosink
  * ]| This example pipeline will decode a WebM stream and decodes the VP9 video.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +44,7 @@
 
 #include <string.h>
 
+#include "gstvpxelements.h"
 #include "gstvp8utils.h"
 #include "gstvp9dec.h"
 
@@ -62,6 +62,7 @@ static gboolean gst_vp9_dec_get_valid_format (GstVPXDec * dec,
     vpx_image_t * img, GstVideoFormat * fmt);
 static void gst_vp9_dec_handle_resolution_change (GstVPXDec * dec,
     vpx_image_t * img, GstVideoFormat fmt);
+static gboolean gst_vp9_dec_get_needs_sync_point (GstVPXDec * dec);
 
 static GstStaticPadTemplate gst_vp9_dec_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -74,11 +75,14 @@ static GstStaticPadTemplate gst_vp9_dec_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("{ I420, YV12, Y42B, Y444, GBR }"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE
+        ("{ I420, YV12, Y42B, Y444, GBR, I420_10LE, I422_10LE }"))
     );
 
 #define parent_class gst_vp9_dec_parent_class
 G_DEFINE_TYPE (GstVP9Dec, gst_vp9_dec, GST_TYPE_VPX_DEC);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (vp9dec, "vp9dec", GST_RANK_PRIMARY,
+    gst_vp9_dec_get_type (), vpx_element_init (plugin));
 
 static void
 gst_vp9_dec_class_init (GstVP9DecClass * klass)
@@ -107,6 +111,8 @@ gst_vp9_dec_class_init (GstVP9DecClass * klass)
       GST_DEBUG_FUNCPTR (gst_vp9_dec_get_valid_format);
   vpx_class->handle_resolution_change =
       GST_DEBUG_FUNCPTR (gst_vp9_dec_handle_resolution_change);
+  vpx_class->get_needs_sync_point =
+      GST_DEBUG_FUNCPTR (gst_vp9_dec_get_needs_sync_point);
 
   GST_DEBUG_CATEGORY_INIT (gst_vp9dec_debug, "vp9dec", 0, "VP9 Decoder");
 }
@@ -158,22 +164,26 @@ gst_vp9_dec_get_valid_format (GstVPXDec * dec, vpx_image_t * img,
           (NULL), ("Unsupported frame format - 4:4:0 planar"));
       return FALSE;
 #endif
-#ifdef VPX_IMG_FMT_I42016
     case VPX_IMG_FMT_I42016:
       /* VPX_IMG_FMT_I420 | VPX_IMG_FMT_HIGHBITDEPTH */
+      if (img->bit_depth == 10) {
+        *fmt = GST_VIDEO_FORMAT_I420_10LE;
+        return TRUE;
+      }
       GST_FIXME_OBJECT (dec, "Please add 16-bit I420 format");
       GST_ELEMENT_WARNING (dec, STREAM, NOT_IMPLEMENTED,
           (NULL), ("Unsupported frame format - 16-bit 4:2:0 planar"));
       return FALSE;
-#endif
-#ifdef VPX_IMG_FMT_I42216
     case VPX_IMG_FMT_I42216:
       /* VPX_IMG_FMT_I422 | VPX_IMG_FMT_HIGHBITDEPTH */
+      if (img->bit_depth == 10) {
+        *fmt = GST_VIDEO_FORMAT_I422_10LE;
+        return TRUE;
+      }
       GST_FIXME_OBJECT (dec, "Please add 16-bit Y42B format");
       GST_ELEMENT_WARNING (dec, STREAM, NOT_IMPLEMENTED,
           (NULL), ("Unsupported frame format - 16-bit 4:2:2 planar"));
       return FALSE;
-#endif
 #ifdef VPX_IMG_FMT_I44416
     case VPX_IMG_FMT_I44416:
       /* VPX_IMG_FMT_I444 | VPX_IMG_FMT_HIGHBITDEPTH */
@@ -219,4 +229,10 @@ gst_vp9_dec_handle_resolution_change (GstVPXDec * dec, vpx_image_t * img,
   }
 }
 
+static gboolean
+gst_vp9_dec_get_needs_sync_point (GstVPXDec * dec)
+{
+  return TRUE;
+}
+
 #endif /* HAVE_VP9_DECODER */
