@@ -43,6 +43,7 @@
 
 #include <gst/audio/audio.h>
 
+#include "gstflvelements.h"
 #include "gstflvmux.h"
 #include "amfdefs.h"
 
@@ -97,6 +98,8 @@ G_DEFINE_TYPE (GstFlvMuxPad, gst_flv_mux_pad, GST_TYPE_AGGREGATOR_PAD);
 #define gst_flv_mux_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstFlvMux, gst_flv_mux, GST_TYPE_AGGREGATOR,
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (flvmux, "flvmux",
+    GST_RANK_PRIMARY, GST_TYPE_FLV_MUX, flv_element_init (plugin));
 
 static GstFlowReturn
 gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout);
@@ -133,7 +136,9 @@ static GstFlowReturn gst_flv_mux_rewrite_header (GstFlvMux * mux);
 static gboolean gst_flv_mux_are_all_pads_eos (GstFlvMux * mux);
 static GstFlowReturn gst_flv_mux_update_src_caps (GstAggregator * aggregator,
     GstCaps * caps, GstCaps ** ret);
-static guint64 gst_flv_mux_query_upstream_duration (GstFlvMux * mux);
+static GstClockTime gst_flv_mux_query_upstream_duration (GstFlvMux * mux);
+static GstClockTime gst_flv_mux_segment_to_running_time (const GstSegment *
+    segment, GstClockTime t);
 
 static GstFlowReturn
 gst_flv_mux_pad_flush (GstAggregatorPad * pad, GstAggregator * aggregator)
@@ -141,12 +146,53 @@ gst_flv_mux_pad_flush (GstAggregatorPad * pad, GstAggregator * aggregator)
   GstFlvMuxPad *flvpad = GST_FLV_MUX_PAD (pad);
 
   flvpad->last_timestamp = GST_CLOCK_TIME_NONE;
-  flvpad->pts = GST_CLOCK_STIME_NONE;
-  flvpad->dts = GST_CLOCK_STIME_NONE;
+  flvpad->pts = GST_CLOCK_TIME_NONE;
+  flvpad->dts = GST_CLOCK_TIME_NONE;
 
   return GST_FLOW_OK;
 }
 
+static gboolean
+gst_flv_mux_skip_buffer (GstAggregatorPad * apad, GstAggregator * aggregator,
+    GstBuffer * buffer)
+{
+  GstFlvMuxPad *fpad = GST_FLV_MUX_PAD_CAST (apad);
+  GstFlvMux *mux = GST_FLV_MUX_CAST (aggregator);
+  GstClockTime t;
+
+  if (!mux->skip_backwards_streams)
+    return FALSE;
+
+  if (fpad->drop_deltas) {
+    if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT)) {
+      GST_INFO_OBJECT (fpad, "Waiting for keyframe, dropping %" GST_PTR_FORMAT,
+          buffer);
+      return TRUE;
+    } else {
+      /* drop-deltas is set and the buffer isn't delta, drop flag */
+      fpad->drop_deltas = FALSE;
+    }
+  }
+
+  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS_OR_PTS (buffer))) {
+    t = gst_flv_mux_segment_to_running_time (&apad->segment,
+        GST_BUFFER_DTS_OR_PTS (buffer));
+
+    if (t < (GST_MSECOND * mux->last_dts)) {
+      GST_WARNING_OBJECT (fpad,
+          "Timestamp %" GST_TIME_FORMAT " going backwards from last used %"
+          GST_TIME_FORMAT ", dropping %" GST_PTR_FORMAT,
+          GST_TIME_ARGS (t), GST_TIME_ARGS (GST_MSECOND * mux->last_dts),
+          buffer);
+      /* Look for non-delta buffer */
+      fpad->drop_deltas = TRUE;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 static void
 gst_flv_mux_pad_class_init (GstFlvMuxPadClass * klass)
 {
@@ -156,6 +202,8 @@ gst_flv_mux_pad_class_init (GstFlvMuxPadClass * klass)
   gobject_class->finalize = gst_flv_mux_pad_finalize;
 
   aggregatorpad_class->flush = GST_DEBUG_FUNCPTR (gst_flv_mux_pad_flush);
+  aggregatorpad_class->skip_buffer =
+      GST_DEBUG_FUNCPTR (gst_flv_mux_skip_buffer);
 }
 
 static void
@@ -340,7 +388,7 @@ gst_flv_mux_reset (GstElement * element)
 
   mux->duration = GST_CLOCK_TIME_NONE;
   mux->new_tags = FALSE;
-  mux->first_timestamp = GST_CLOCK_STIME_NONE;
+  mux->first_timestamp = GST_CLOCK_TIME_NONE;
   mux->last_dts = 0;
 
   mux->state = GST_FLV_MUX_STATE_HEADER;
@@ -736,9 +784,10 @@ static void
 gst_flv_mux_release_pad (GstElement * element, GstPad * pad)
 {
   GstFlvMux *mux = GST_FLV_MUX (element);
-  GstFlvMuxPad *flvpad = GST_FLV_MUX_PAD (pad);
+  GstFlvMuxPad *flvpad = GST_FLV_MUX_PAD (gst_object_ref (pad));
+
+  GST_ELEMENT_CLASS (gst_flv_mux_parent_class)->release_pad (element, pad);
 
-  gst_pad_set_active (pad, FALSE);
   gst_flv_mux_reset_pad (flvpad);
 
   if (flvpad == mux->video_pad) {
@@ -749,7 +798,7 @@ gst_flv_mux_release_pad (GstElement * element, GstPad * pad)
     GST_WARNING_OBJECT (pad, "Pad is not known audio or video pad");
   }
 
-  gst_element_remove_pad (element, pad);
+  gst_object_unref (flvpad);
 }
 
 static GstFlowReturn
@@ -856,7 +905,7 @@ gst_flv_mux_create_metadata (GstFlvMux * mux)
   dts = mux->last_dts;
 
   /* Timestamp must start at zero */
-  if (GST_CLOCK_STIME_IS_VALID (mux->first_timestamp)) {
+  if (GST_CLOCK_TIME_IS_VALID (mux->first_timestamp)) {
     dts -= mux->first_timestamp / GST_MSECOND;
   }
 
@@ -925,7 +974,7 @@ gst_flv_mux_create_metadata (GstFlvMux * mux)
   for (i = 0; tags && i < n_tags; i++) {
     const gchar *tag_name = gst_tag_list_nth_tag_name (tags, i);
     if (!strcmp (tag_name, GST_TAG_DURATION)) {
-      guint64 dur;
+      GstClockTime dur;
 
       if (!gst_tag_list_get_uint64 (tags, GST_TAG_DURATION, &dur))
         continue;
@@ -1169,13 +1218,28 @@ gst_flv_mux_buffer_to_tag_internal (GstFlvMux * mux, GstBuffer * buffer,
   guint8 *data, *bdata = NULL;
   gsize bsize = 0;
 
-  if (GST_CLOCK_STIME_IS_VALID (pad->dts)) {
+  if (GST_CLOCK_TIME_IS_VALID (pad->dts)) {
     pts = pad->pts / GST_MSECOND;
     dts = pad->dts / GST_MSECOND;
+    GST_LOG_OBJECT (mux,
+        "Pad %s: Created dts %" GST_TIME_FORMAT ", pts %" GST_TIME_FORMAT
+        " from rounding %" GST_TIME_FORMAT ", %" GST_TIME_FORMAT,
+        GST_PAD_NAME (pad), GST_TIME_ARGS (dts * GST_MSECOND),
+        GST_TIME_ARGS (pts * GST_MSECOND), GST_TIME_ARGS (pad->dts),
+        GST_TIME_ARGS (pad->pts));
   } else if (GST_CLOCK_TIME_IS_VALID (pad->last_timestamp)) {
     pts = dts = pad->last_timestamp / GST_MSECOND;
+    GST_DEBUG_OBJECT (mux,
+        "Pad %s: Created dts and pts %" GST_TIME_FORMAT
+        " from rounding last pad timestamp %" GST_TIME_FORMAT,
+        GST_PAD_NAME (pad), GST_TIME_ARGS (pts * GST_MSECOND),
+        GST_TIME_ARGS (pad->last_timestamp));
   } else {
     pts = dts = mux->last_dts;
+    GST_DEBUG_OBJECT (mux,
+        "Pad %s: Created dts and pts %" GST_TIME_FORMAT
+        " from last mux timestamp",
+        GST_PAD_NAME (pad), GST_TIME_ARGS (pts * GST_MSECOND));
   }
 
   /* We prevent backwards timestamps because they confuse librtmp,
@@ -1197,7 +1261,7 @@ gst_flv_mux_buffer_to_tag_internal (GstFlvMux * mux, GstBuffer * buffer,
     cts = 0;
 
   /* Timestamp must start at zero */
-  if (GST_CLOCK_STIME_IS_VALID (mux->first_timestamp)) {
+  if (GST_CLOCK_TIME_IS_VALID (mux->first_timestamp)) {
     dts -= mux->first_timestamp / GST_MSECOND;
     pts = dts + cts;
   }
@@ -1279,7 +1343,7 @@ gst_flv_mux_buffer_to_tag_internal (GstFlvMux * mux, GstBuffer * buffer,
     data[11] |= (pad->width << 1) & 0x02;
     data[11] |= (pad->channels << 0) & 0x01;
 
-    GST_DEBUG_OBJECT (mux, "Creating byte %02x with "
+    GST_LOG_OBJECT (mux, "Creating byte %02x with "
         "codec:%d, rate:%d, width:%d, channels:%d",
         data[11], pad->codec, pad->rate, pad->width, pad->channels);
 
@@ -1654,7 +1718,7 @@ static gboolean
 duration_query_cb (GstElement * element, GstPad * pad,
     struct DurationData *data)
 {
-  guint64 dur;
+  GstClockTime dur;
 
   if (gst_pad_peer_query_duration (GST_PAD (pad), GST_FORMAT_TIME,
           (gint64 *) & dur) && dur != GST_CLOCK_TIME_NONE) {
@@ -1667,7 +1731,7 @@ duration_query_cb (GstElement * element, GstPad * pad,
   return TRUE;
 }
 
-static guint64
+static GstClockTime
 gst_flv_mux_query_upstream_duration (GstFlvMux * mux)
 {
   struct DurationData cb_data = { GST_CLOCK_TIME_NONE };
@@ -1853,7 +1917,6 @@ static GstFlvMuxPad *
 gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
     gboolean timeout)
 {
-  GstFlvMux *mux = GST_FLV_MUX (aggregator);
   GstFlvMuxPad *best = NULL;
   GstClockTime best_ts = GST_CLOCK_TIME_NONE;
   GstIterator *pads;
@@ -1866,7 +1929,6 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
     switch (gst_iterator_next (pads, &padptr)) {
       case GST_ITERATOR_OK:{
         GstAggregatorPad *apad = g_value_get_object (&padptr);
-        GstFlvMuxPad *fpad = GST_FLV_MUX_PAD (apad);
         GstClockTime t = GST_CLOCK_TIME_NONE;
         GstBuffer *buffer;
 
@@ -1880,46 +1942,9 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
           break;
         }
 
-        if (fpad->drop_deltas) {
-          if (mux->skip_backwards_streams
-              && GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT)) {
-            GST_INFO_OBJECT (apad,
-                "Dropped buffer %" GST_PTR_FORMAT " until keyframe", buffer);
-            gst_buffer_unref (buffer);
-            gst_aggregator_pad_drop_buffer (apad);
-            if (!timeout) {
-              gst_object_replace ((GstObject **) & best, NULL);
-              best_ts = GST_CLOCK_TIME_NONE;
-              done = TRUE;
-            }
-            break;
-          } else {
-            /* drop-deltas is set and the buffer isn't delta, drop flag */
-            fpad->drop_deltas = FALSE;
-          }
-        }
-
         if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS_OR_PTS (buffer))) {
           t = gst_flv_mux_segment_to_running_time (&apad->segment,
               GST_BUFFER_DTS_OR_PTS (buffer));
-
-          if (mux->skip_backwards_streams
-              && (t < (GST_MSECOND * mux->last_dts))) {
-            GST_WARNING_OBJECT (mux,
-                "Timestamp %" GST_TIME_FORMAT " going "
-                "backwards from last known %" GST_TIME_FORMAT ", dropping",
-                GST_TIME_ARGS (t), GST_TIME_ARGS (GST_MSECOND * mux->last_dts));
-            gst_buffer_unref (buffer);
-            gst_aggregator_pad_drop_buffer (apad);
-            /* Look for non-delta buffer */
-            fpad->drop_deltas = TRUE;
-            if (!timeout) {
-              gst_object_replace ((GstObject **) & best, NULL);
-              best_ts = GST_CLOCK_TIME_NONE;
-              done = TRUE;
-            }
-            break;
-          }
         }
 
         if (!GST_CLOCK_TIME_IS_VALID (best_ts) ||
@@ -1967,7 +1992,7 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
 {
   GstFlvMux *mux = GST_FLV_MUX (aggregator);
   GstFlvMuxPad *best;
-  gint64 best_time = GST_CLOCK_STIME_NONE;
+  GstClockTime best_time = GST_CLOCK_TIME_NONE;
   GstFlowReturn ret;
   GstClockTime ts;
   GstBuffer *buffer = NULL;
@@ -1995,8 +2020,8 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
 
     mux->state = GST_FLV_MUX_STATE_DATA;
 
-    if (!mux->streamable || mux->first_timestamp == GST_CLOCK_STIME_NONE) {
-      if (best && GST_CLOCK_STIME_IS_VALID (ts))
+    if (!mux->streamable || mux->first_timestamp == GST_CLOCK_TIME_NONE) {
+      if (best && GST_CLOCK_TIME_IS_VALID (ts))
         mux->first_timestamp = ts;
       else
         mux->first_timestamp = 0;
@@ -2026,7 +2051,7 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
         gst_flv_mux_segment_to_running_time (&GST_AGGREGATOR_PAD
         (best)->segment, GST_BUFFER_DTS_OR_PTS (buffer));
 
-    if (GST_CLOCK_STIME_IS_VALID (best->dts))
+    if (GST_CLOCK_TIME_IS_VALID (best->dts))
       best_time = best->dts - mux->first_timestamp;
 
     if (GST_BUFFER_PTS_IS_VALID (buffer))
@@ -2036,9 +2061,9 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
     else
       best->pts = best->dts;
 
-    GST_LOG_OBJECT (best, "got buffer PTS %" GST_TIME_FORMAT " DTS %"
-        GST_STIME_FORMAT, GST_TIME_ARGS (best->pts),
-        GST_STIME_ARGS (best->dts));
+    GST_LOG_OBJECT (best,
+        "got buffer PTS %" GST_TIME_FORMAT " DTS %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (best->pts), GST_TIME_ARGS (best->dts));
   } else {
     if (!gst_flv_mux_are_all_pads_eos (mux))
       return GST_AGGREGATOR_FLOW_NEED_DATA;
@@ -2048,7 +2073,7 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
   /* The FLV timestamp is an int32 field. For non-live streams error out if a
      bigger timestamp is seen, for live the timestamp will get wrapped in
      gst_flv_mux_buffer_to_tag */
-  if (!mux->streamable && (GST_CLOCK_STIME_IS_VALID (best_time))
+  if (!mux->streamable && (GST_CLOCK_TIME_IS_VALID (best_time))
       && best_time / GST_MSECOND > G_MAXINT32) {
     GST_WARNING_OBJECT (mux, "Timestamp larger than FLV supports - EOS");
     if (buffer) {
