@@ -303,6 +303,51 @@ qtdemux_dump_stsd_avc1 (GstQTDemux * qtdemux, GstByteReader * data, guint size,
   return TRUE;
 }
 
+
+static gboolean
+qtdemux_dump_stsd_av01 (GstQTDemux * qtdemux, GstByteReader * data, guint size,
+    int depth)
+{
+  guint compressor_len;
+  char compressor_name[32];
+
+  /* Size of av01 = 78 bytes */
+  if (size < (6 + 2 + 4 + 12 + 2 + 2 + 4 + 4 + 4 + 2 + 1 + 31 + 2 + 2))
+    return FALSE;
+
+  gst_byte_reader_skip_unchecked (data, 6);
+  GST_LOG_OBJECT (qtdemux, "%*s    data reference:%d", depth, "",
+      GET_UINT16 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    version/rev.:  %08x", depth, "",
+      GET_UINT32 (data));
+  gst_byte_reader_skip_unchecked (data, 12);    /* pre-defined & reserved */
+  GST_LOG_OBJECT (qtdemux, "%*s    width:         %u", depth, "",
+      GET_UINT16 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    height:        %u", depth, "",
+      GET_UINT16 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    horiz. resol:  %g", depth, "",
+      GET_FP32 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    vert. resol.:  %g", depth, "",
+      GET_FP32 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    data size:     %u", depth, "",
+      GET_UINT32 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    frame count:   %u", depth, "",
+      GET_UINT16 (data));
+  /* something is not right with this, it's supposed to be a string but it's
+   * not apparently, so just skip this for now */
+  compressor_len = MAX (GET_UINT8 (data), 31);
+  memcpy (compressor_name, gst_byte_reader_get_data_unchecked (data, 31), 31);
+  compressor_name[compressor_len] = 0;
+  GST_LOG_OBJECT (qtdemux, "%*s    compressor:    %s", depth, "",
+      compressor_name);
+  GST_LOG_OBJECT (qtdemux, "%*s    depth:         %u", depth, "",
+      GET_UINT16 (data));
+  GST_LOG_OBJECT (qtdemux, "%*s    color table ID:%u", depth, "",
+      GET_UINT16 (data));
+
+  return TRUE;
+}
+
 gboolean
 qtdemux_dump_stsd (GstQTDemux * qtdemux, GstByteReader * data, int depth)
 {
@@ -353,6 +398,10 @@ qtdemux_dump_stsd (GstQTDemux * qtdemux, GstByteReader * data, int depth)
         if (!qtdemux_dump_unknown (qtdemux, &sub, depth + 1))
           return FALSE;
         break;
+      case FOURCC_av01:
+        if (!qtdemux_dump_stsd_av01 (qtdemux, &sub, size, depth + 1))
+          return FALSE;
+        break;
       default:
         /* Unknown stsd data, dump the bytes */
         if (!qtdemux_dump_unknown (qtdemux, &sub, depth + 1))
@@ -528,21 +577,37 @@ qtdemux_dump_ctts (GstQTDemux * qtdemux, GstByteReader * data, int depth)
 gboolean
 qtdemux_dump_cslg (GstQTDemux * qtdemux, GstByteReader * data, int depth)
 {
-  guint32 ver_flags = 0, shift = 0;
-  gint32 least_offset = 0, start_time = 0, end_time = 0;
+  guint32 ver_flags = 0;
 
-  if (!gst_byte_reader_get_uint32_be (data, &ver_flags) ||
-      !gst_byte_reader_get_uint32_be (data, &shift) ||
-      !gst_byte_reader_get_int32_be (data, &least_offset) ||
-      !gst_byte_reader_get_int32_be (data, &start_time) ||
-      !gst_byte_reader_get_int32_be (data, &end_time))
+  if (!gst_byte_reader_get_uint32_be (data, &ver_flags))
     return FALSE;
 
   GST_LOG ("%*s  version/flags: %08x", depth, "", ver_flags);
-  GST_LOG ("%*s  shift:         %u", depth, "", shift);
-  GST_LOG ("%*s  least offset:  %d", depth, "", least_offset);
-  GST_LOG ("%*s  start time:    %d", depth, "", start_time);
-  GST_LOG ("%*s  end time:      %d", depth, "", end_time);
+
+  if (ver_flags >> 24 == 0) {
+    gint32 shift = 0, least_offset = 0, start_time = 0, end_time = 0;
+    if (!gst_byte_reader_get_int32_be (data, &shift) ||
+        !gst_byte_reader_get_int32_be (data, &least_offset) ||
+        !gst_byte_reader_get_int32_be (data, &start_time) ||
+        !gst_byte_reader_get_int32_be (data, &end_time))
+      return FALSE;
+    GST_LOG ("%*s  shift:         %d", depth, "", shift);
+    GST_LOG ("%*s  least offset:  %d", depth, "", least_offset);
+    GST_LOG ("%*s  start time:    %d", depth, "", start_time);
+    GST_LOG ("%*s  end time:      %d", depth, "", end_time);
+  } else {
+    gint64 shift = 0, least_offset = 0, start_time = 0, end_time = 0;
+    if (!gst_byte_reader_get_int64_be (data, &shift) ||
+        !gst_byte_reader_get_int64_be (data, &least_offset) ||
+        !gst_byte_reader_get_int64_be (data, &start_time) ||
+        !gst_byte_reader_get_int64_be (data, &end_time))
+      return FALSE;
+
+    GST_LOG ("%*s  shift:         %" G_GINT64_FORMAT, depth, "", shift);
+    GST_LOG ("%*s  least offset:  %" G_GINT64_FORMAT, depth, "", least_offset);
+    GST_LOG ("%*s  start time:    %" G_GINT64_FORMAT, depth, "", start_time);
+    GST_LOG ("%*s  end time:      %" G_GINT64_FORMAT, depth, "", end_time);
+  }
 
   return TRUE;
 }
@@ -763,7 +828,7 @@ qtdemux_dump_trun (GstQTDemux * qtdemux, GstByteReader * data, int depth)
     if (flags & TR_COMPOSITION_TIME_OFFSETS) {
       if (!gst_byte_reader_get_uint32_be (data, &composition_time_offsets))
         return FALSE;
-      GST_TRACE ("%*s    composition_time_offsets:  %u", depth, "",
+      GST_TRACE ("%*s    composition_time_offsets:  %d", depth, "",
           composition_time_offsets);
     }
   }
