@@ -87,6 +87,10 @@ struct _GstMultipartDemux
   gint scanpos;
 
   gboolean singleStream;
+
+  /* to handle stream-start */
+  gboolean have_group_id;
+  guint group_id;
 };
 
 struct _GstMultipartDemuxClass
@@ -98,7 +102,7 @@ struct _GstMultipartDemuxClass
 
 GType gst_multipart_demux_get_type (void);
 
-gboolean gst_multipart_demux_plugin_init (GstPlugin * plugin);
+GST_ELEMENT_REGISTER_DECLARE (multipartdemux);
 
 G_END_DECLS
 
