@@ -19,18 +19,18 @@
 
 /**
  * SECTION:element-rtpamrpay
+ * @title: rtpamrpay
  * @see_also: rtpamrdepay
  *
  * Payload AMR audio into RTP packets according to RFC 3267.
  * For detailed information see: http://www.rfc-editor.org/rfc/rfc3267.txt
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 -v audiotestsrc ! amrnbenc ! rtpamrpay ! udpsink
  * ]| This example pipeline will encode and payload an AMR stream. Refer to
  * the rtpamrdepay example to depayload and decode the RTP stream.
- * </refsect2>
+ *
  */
 
 /* references:
@@ -55,6 +55,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/audio/audio.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpamrpay.h"
 #include "gstrtputils.h"
 
@@ -113,6 +114,8 @@ gst_rtp_amr_pay_change_state (GstElement * element, GstStateChange transition);
 
 #define gst_rtp_amr_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpAMRPay, gst_rtp_amr_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpamrpay, "rtpamrpay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_AMR_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_amr_pay_class_init (GstRtpAMRPayClass * klass)
@@ -312,7 +315,9 @@ gst_rtp_amr_pay_handle_buffer (GstRTPBasePayload * basepayload,
     goto too_big;
 
   /* now alloc output buffer */
-  outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
+  outbuf =
+      gst_rtp_base_payload_allocate_output_buffer (basepayload, payload_len, 0,
+      0);
 
   gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
 
@@ -454,10 +459,3 @@ gst_rtp_amr_pay_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
-
-gboolean
-gst_rtp_amr_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpamrpay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_AMR_PAY);
-}
