@@ -26,6 +26,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/audio/audio.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpmp4apay.h"
 #include "gstrtputils.h"
 
@@ -68,6 +69,8 @@ static GstFlowReturn gst_rtp_mp4a_pay_handle_buffer (GstRTPBasePayload *
 
 #define gst_rtp_mp4a_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpMP4APay, gst_rtp_mp4a_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpmp4apay, "rtpmp4apay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_MP4A_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_mp4a_pay_class_init (GstRtpMP4APayClass * klass)
@@ -456,10 +459,3 @@ gst_rtp_mp4a_pay_handle_buffer (GstRTPBasePayload * basepayload,
 
   return ret;
 }
-
-gboolean
-gst_rtp_mp4a_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpmp4apay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_MP4A_PAY);
-}
