@@ -19,22 +19,22 @@
 
 /**
  * SECTION:element-shapewipe
+ * @title: shapewipe
  *
  * The shapewipe element provides custom transitions on video streams
  * based on a grayscale bitmap. The state of the transition can be
  * controlled by the position property and an optional blended border
  * can be added by the border property.
  *
- * Transition bitmaps can be downloaded from the
- * <ulink url="http://cinelerra.org/transitions.php">Cinelerra transition</ulink>
- * page.
+ * Transition bitmaps can be downloaded from the Cinelerra pages
+ * [here](http://cinelerra-cv.wikidot.com/main:transitions-themes) or
+ * [here](https://cinelerra-gg.org/download/CinelerraGG_Manual/Shape_Wipe.html).
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v videotestsrc ! video/x-raw,format=AYUV,width=640,height=480 ! shapewipe position=0.5 name=shape ! videomixer name=mixer ! videoconvert ! autovideosink     filesrc location=mask.png ! typefind ! decodebin ! videoconvert ! videoscale ! queue ! shape.mask_sink    videotestsrc pattern=snow ! video/x-raw,format=AYUV,width=640,height=480 ! queue ! mixer.
  * ]| This pipeline adds the transition from mask.png with position 0.5 to an SMPTE test screen and snow.
- * </refsect2>
+ *
  */
 
 
@@ -127,6 +127,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_shape_wipe_debug);
 
 #define gst_shape_wipe_parent_class parent_class
 G_DEFINE_TYPE (GstShapeWipe, gst_shape_wipe, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (shapewipe, "shapewipe", GST_RANK_NONE,
+    GST_TYPE_SHAPE_WIPE, GST_DEBUG_CATEGORY_INIT (gst_shape_wipe_debug,
+        "shapewipe", 0, "shapewipe element"););
 
 static void
 gst_shape_wipe_class_init (GstShapeWipeClass * klass)
@@ -156,12 +159,12 @@ gst_shape_wipe_class_init (GstShapeWipeClass * klass)
       "Adds a shape wipe transition to a video stream",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&video_sink_pad_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&mask_sink_pad_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_pad_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &video_sink_pad_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &mask_sink_pad_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &src_pad_template);
 }
 
 static void
@@ -329,8 +332,9 @@ gst_shape_wipe_video_sink_getcaps (GstShapeWipe * self, GstPad * pad,
 {
   GstCaps *templ, *ret, *tmp;
 
-  if (gst_pad_has_current_caps (pad))
-    return gst_pad_get_current_caps (pad);
+  ret = gst_pad_get_current_caps (pad);
+  if (ret != NULL)
+    return ret;
 
   templ = gst_pad_get_pad_template_caps (pad);
   tmp = gst_pad_peer_query_caps (self->srcpad, NULL);
@@ -451,8 +455,9 @@ gst_shape_wipe_mask_sink_getcaps (GstShapeWipe * self, GstPad * pad,
   GstCaps *ret, *tmp, *tcaps;
   guint i, n;
 
-  if (gst_pad_has_current_caps (pad))
-    return gst_pad_get_current_caps (pad);
+  ret = gst_pad_get_current_caps (pad);
+  if (ret != NULL)
+    return ret;
 
   tcaps = gst_pad_get_pad_template_caps (self->video_sinkpad);
   tmp = gst_pad_peer_query_caps (self->video_sinkpad, NULL);
@@ -535,10 +540,13 @@ gst_shape_wipe_src_getcaps (GstPad * pad, GstCaps * filter)
   GstShapeWipe *self = GST_SHAPE_WIPE (gst_pad_get_parent (pad));
   GstCaps *templ, *ret, *tmp;
 
-  if (gst_pad_has_current_caps (pad))
-    return gst_pad_get_current_caps (pad);
-  else if (gst_pad_has_current_caps (self->video_sinkpad))
-    return gst_pad_get_current_caps (self->video_sinkpad);
+  ret = gst_pad_get_current_caps (pad);
+  if (ret != NULL)
+    return ret;
+
+  ret = gst_pad_get_current_caps (self->video_sinkpad);
+  if (ret != NULL)
+    return ret;
 
   templ = gst_pad_get_pad_template_caps (self->video_sinkpad);
   tmp = gst_pad_peer_query_caps (self->video_sinkpad, NULL);
@@ -1128,14 +1136,7 @@ gst_shape_wipe_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (gst_shape_wipe_debug, "shapewipe", 0,
-      "shapewipe element");
-
-  if (!gst_element_register (plugin, "shapewipe", GST_RANK_NONE,
-          GST_TYPE_SHAPE_WIPE))
-    return FALSE;
-
-  return TRUE;
+  return GST_ELEMENT_REGISTER (shapewipe, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
