@@ -340,43 +340,116 @@ gst_core_audio_parse_channel_layout (AudioChannelLayout * layout,
   g_assert (channel_mask != NULL);
   g_assert (layout != NULL);
 
-  if (layout->mChannelLayoutTag !=
+  if (layout->mChannelLayoutTag ==
       kAudioChannelLayoutTag_UseChannelDescriptions) {
-    GST_ERROR
-        ("Only kAudioChannelLayoutTag_UseChannelDescriptions is supported.");
+
+    switch (layout->mNumberChannelDescriptions) {
+      case 0:
+        if (pos)
+          pos[0] = GST_AUDIO_CHANNEL_POSITION_NONE;
+        *channels = 0;
+        *channel_mask = 0;
+        return TRUE;
+      case 1:
+        if (pos)
+          pos[0] = GST_AUDIO_CHANNEL_POSITION_MONO;
+        *channels = 1;
+        *channel_mask = 0;
+        return TRUE;
+      case 2:
+        if (pos) {
+          pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+          pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        }
+        *channels = 2;
+        *channel_mask =
+            GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+            GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT);
+        return TRUE;
+      default:
+        _core_audio_parse_channel_descriptions (layout, channels, channel_mask,
+            pos);
+        return TRUE;
+    }
+  } else if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_Mono) {
+    if (pos)
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_MONO;
+    *channels = 1;
+    *channel_mask = 0;
+    return TRUE;
+  } else if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_Stereo ||
+      layout->mChannelLayoutTag == kAudioChannelLayoutTag_StereoHeadphones ||
+      layout->mChannelLayoutTag == kAudioChannelLayoutTag_Binaural) {
+    if (pos) {
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+      pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+    }
+    *channels = 2;
+    *channel_mask =
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT);
+    return TRUE;
+  } else if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_Quadraphonic) {
+    if (pos) {
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+      pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      pos[2] = GST_AUDIO_CHANNEL_POSITION_SURROUND_LEFT;
+      pos[3] = GST_AUDIO_CHANNEL_POSITION_SURROUND_RIGHT;
+    }
+    *channels = 4;
+    *channel_mask =
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (SURROUND_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (SURROUND_RIGHT);
+    return TRUE;
+  } else if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_Pentagonal) {
+    if (pos) {
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+      pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      pos[2] = GST_AUDIO_CHANNEL_POSITION_SURROUND_LEFT;
+      pos[3] = GST_AUDIO_CHANNEL_POSITION_SURROUND_RIGHT;
+      pos[4] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
+    }
+    *channels = 5;
+    *channel_mask =
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (SURROUND_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (SURROUND_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_CENTER);
+    return TRUE;
+  } else if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_Cube) {
+    if (pos) {
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+      pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      pos[2] = GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+      pos[3] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+      pos[4] = GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT;
+      pos[5] = GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT;
+      pos[6] = GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT;
+      pos[7] = GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT;
+
+    }
+    *channels = 8;
+    *channel_mask =
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (REAR_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (REAR_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (TOP_FRONT_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (TOP_FRONT_RIGHT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (TOP_REAR_LEFT) |
+        GST_AUDIO_CHANNEL_POSITION_MASK (TOP_REAR_RIGHT);
+    return TRUE;
+  } else {
+    GST_WARNING
+        ("AudioChannelLayoutTag: %u not yet supported",
+        layout->mChannelLayoutTag);
     *channels = 0;
     *channel_mask = 0;
     return FALSE;
   }
-
-  switch (layout->mNumberChannelDescriptions) {
-    case 0:
-      if (pos)
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_NONE;
-      *channels = 0;
-      *channel_mask = 0;
-      return TRUE;
-    case 1:
-      if (pos)
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_MONO;
-      *channels = 1;
-      *channel_mask = 0;
-      return TRUE;
-    case 2:
-      if (pos) {
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
-        pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
-      }
-      *channels = 2;
-      *channel_mask =
-          GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
-          GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT);
-      return TRUE;
-    default:
-      _core_audio_parse_channel_descriptions (layout, channels, channel_mask,
-          pos);
-      return TRUE;
-  }
 }
 
 /* Converts an AudioStreamBasicDescription to preferred caps.
@@ -405,6 +478,7 @@ gst_core_audio_asbd_to_caps (AudioStreamBasicDescription * asbd,
   guint rate, channels, bps, endianness;
   guint64 channel_mask;
   gboolean sign, interleaved;
+  GstAudioChannelPosition pos[GST_OSX_AUDIO_MAX_CHANNEL];
 
   if (asbd->mFormatID != kAudioFormatLinearPCM) {
     GST_WARNING ("Only linear PCM is supported");
@@ -430,7 +504,7 @@ gst_core_audio_asbd_to_caps (AudioStreamBasicDescription * asbd,
   bps = asbd->mBitsPerChannel;
   endianness = asbd->mFormatFlags & kAudioFormatFlagIsBigEndian ?
       G_BIG_ENDIAN : G_LITTLE_ENDIAN;
-  sign = asbd->mFormatID & kAudioFormatFlagIsSignedInteger ? TRUE : FALSE;
+  sign = asbd->mFormatFlags & kAudioFormatFlagIsSignedInteger ? TRUE : FALSE;
   interleaved = asbd->mFormatFlags & kAudioFormatFlagIsNonInterleaved ?
       TRUE : FALSE;
 
@@ -457,14 +531,15 @@ gst_core_audio_asbd_to_caps (AudioStreamBasicDescription * asbd,
   }
 
   if (layout) {
-    GstAudioChannelPosition pos[GST_OSX_AUDIO_MAX_CHANNEL];
-
     if (!gst_core_audio_parse_channel_layout (layout, &channels, &channel_mask,
             pos)) {
-      GST_WARNING ("Failed to parse channel layout");
-      goto error;
+      GST_WARNING
+          ("Failed to parse channel layout, best effort channels layout mapping will be used");
+      layout = NULL;
     }
+  }
 
+  if (layout) {
     /* The AU can have arbitrary channel order, but we're using GstAudioInfo
      * which supports only the GStreamer channel order.
      * Also, we're eventually producing caps, which only have channel-mask
@@ -563,7 +638,7 @@ gst_core_audio_probe_caps (GstCoreAudio * core_audio, GstCaps * in_caps)
   got_outer_asbd =
       _core_audio_get_stream_format (core_audio, &outer_asbd, TRUE);
 
-  /* Collect info about the HW capabilites and preferences */
+  /* Collect info about the HW capabilities and preferences */
   spdif_allowed =
       gst_core_audio_audio_device_is_spdif_avail (core_audio->device_id);
   if (!core_audio->is_src)
@@ -644,24 +719,33 @@ gst_core_audio_probe_caps (GstCoreAudio * core_audio, GstCaps * in_caps)
        * e.g. if you push 5.1-surround audio to a stereo configuration,
        * the left and right channels will be played accordingly,
        * and the rest will be dropped. */
-
-      if (channels == 1 || (channels == 2 &&
-              (channel_mask == 0 || channel_mask == STEREO_CHANNEL_MASK))) {
-
+      if (channels == 1) {
+        /* If have mono, then also offer stereo since CoreAudio downmixes to it */
+        GstStructure *stereo = gst_structure_copy (out_s);
+        gst_structure_remove_field (out_s, "channel-mask");
+        gst_structure_set (stereo, "channels", G_TYPE_INT, 2,
+            "channel-mask", GST_TYPE_BITMASK, STEREO_CHANNEL_MASK, NULL);
+        gst_caps_append_structure (caps, stereo);
+        gst_caps_append_structure (caps, out_s);
+      } else if (channels == 2 && (channel_mask == 0
+              || channel_mask == STEREO_CHANNEL_MASK)) {
         /* If have stereo channels, then also offer mono since CoreAudio
-         * upmixes it. If mono, then also offer stereo since CoreAudio
-         * downmixes to it */
+         * upmixes it. */
+        GstStructure *mono = gst_structure_copy (out_s);
+        gst_structure_set (mono, "channels", G_TYPE_INT, 1, NULL);
+        gst_structure_remove_field (mono, "channel-mask");
+        gst_structure_set (out_s, "channel-mask", GST_TYPE_BITMASK,
+            STEREO_CHANNEL_MASK, NULL);
 
-        gst_structure_set (out_s, "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
-
-        if (channels == 1)
-          gst_structure_set (out_s, "channel-mask", GST_TYPE_BITMASK,
-              STEREO_CHANNEL_MASK, NULL);
+        gst_caps_append_structure (caps, out_s);
+        gst_caps_append_structure (caps, mono);
+      } else {
+        /* Otherwise just add the caps */
+        gst_caps_append_structure (caps, out_s);
       }
-
-      gst_caps_append_structure (caps, out_s);
     }
   }
 
+  GST_DEBUG_OBJECT (core_audio, "Probed caps:%" GST_PTR_FORMAT, caps);
   return caps;
 }
