@@ -525,8 +525,8 @@ gst_osx_video_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_static_metadata (element_class, "OSX Video sink",
-      "Sink/Video", "OSX native videosink",
+  gst_element_class_set_static_metadata (element_class, "macOS Video sink",
+      "Sink/Video", "macOS native videosink",
       "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
   gst_element_class_add_static_pad_template (element_class, &gst_osx_video_sink_sink_template_factory);
@@ -573,24 +573,10 @@ gst_osx_video_sink_class_init (GstOSXVideoSinkClass * klass)
   gstbasesink_class->propose_allocation = gst_osx_video_sink_propose_allocation;
   gstelement_class->change_state = gst_osx_video_sink_change_state;
 
-  /**
-   * GstOSXVideoSink:embed
-   *
-   * For ABI comatibility onyl, do not use
-   *
-   **/
-
   g_object_class_install_property (gobject_class, ARG_EMBED,
-      g_param_spec_boolean ("embed", "embed", "For ABI compatiblity only, do not use",
+      g_param_spec_boolean ("embed", "embed", "For ABI compatibility only, do not use",
           FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstOSXVideoSink:force-aspect-ratio
-   *
-   * When enabled, scaling will respect original aspect ratio.
-   *
-   **/
-
   g_object_class_install_property (gobject_class, ARG_FORCE_PAR,
       g_param_spec_boolean ("force-aspect-ratio", "force aspect ration",
           "When enabled, scaling will respect original aspect ration",
@@ -748,7 +734,7 @@ gst_osx_video_sink_get_type (void)
 
 - (void)windowWillClose:(NSNotification *)notification {
   /* Only handle close events if the window was closed manually by the user
-   * and not becuase of a state change state to READY */
+   * and not because of a state change state to READY */
   if (osxvideosink->osxwindow == NULL) {
     return;
   }
