@@ -54,6 +54,7 @@
 #include <gst/video/gstvideometa.h>
 #include <gst/video/gstvideopool.h>
 
+#include "gstv4l2elements.h"
 #include "gstv4l2src.h"
 
 #include "gstv4l2colorbalance.h"
@@ -98,6 +99,16 @@ G_DEFINE_TYPE_WITH_CODE (GstV4l2Src, gst_v4l2src, GST_TYPE_PUSH_SRC,
         gst_v4l2src_color_balance_interface_init);
     G_IMPLEMENT_INTERFACE (GST_TYPE_VIDEO_ORIENTATION,
         gst_v4l2src_video_orientation_interface_init));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (v4l2src,
+    "v4l2src", GST_RANK_PRIMARY, GST_TYPE_V4L2SRC, v4l2_element_init (plugin));
+
+struct PreferredCapsInfo
+{
+  gint width;
+  gint height;
+  gint fps_n;
+  gint fps_d;
+};
 
 static void gst_v4l2src_finalize (GstV4l2Src * v4l2src);
 
@@ -116,7 +127,7 @@ static gboolean gst_v4l2src_decide_allocation (GstBaseSrc * src,
     GstQuery * query);
 static GstFlowReturn gst_v4l2src_create (GstPushSrc * src, GstBuffer ** out);
 static GstCaps *gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps,
-    GstStructure * pref_s);
+    struct PreferredCapsInfo *pref);
 static gboolean gst_v4l2src_negotiate (GstBaseSrc * basesrc);
 
 static void gst_v4l2src_set_property (GObject * object, guint prop_id,
@@ -248,14 +259,6 @@ gst_v4l2src_get_property (GObject * object,
   }
 }
 
-struct PreferedCapsInfo
-{
-  gint width;
-  gint height;
-  gint fps_n;
-  gint fps_d;
-};
-
 static gboolean
 gst_vl42_src_fixate_fields (GQuark field_id, GValue * value, gpointer user_data)
 {
@@ -274,7 +277,7 @@ gst_vl42_src_fixate_fields (GQuark field_id, GValue * value, gpointer user_data)
 
 static void
 gst_v4l2_src_fixate_struct_with_preference (GstStructure * s,
-    struct PreferedCapsInfo *pref)
+    struct PreferredCapsInfo *pref)
 {
   if (gst_structure_has_field (s, "width"))
     gst_structure_fixate_field_nearest_int (s, "width", pref->width);
@@ -308,7 +311,7 @@ gst_v4l2_src_parse_fixed_struct (GstStructure * s,
 /* TODO Consider framerate */
 static gint
 gst_v4l2src_fixed_caps_compare (GstCaps * caps_a, GstCaps * caps_b,
-    struct PreferedCapsInfo *pref)
+    struct PreferredCapsInfo *pref)
 {
   GstStructure *a, *b;
   gint aw = G_MAXINT, ah = G_MAXINT, ad = G_MAXINT;
@@ -389,12 +392,9 @@ gst_v4l2src_set_format (GstV4l2Src * v4l2src, GstCaps * caps,
 }
 
 static GstCaps *
-gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps, GstStructure * pref_s)
+gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps,
+    struct PreferredCapsInfo *pref)
 {
-  /* Let's prefer a good resolutiion as of today's standard. */
-  struct PreferedCapsInfo pref = {
-    3840, 2160, 120, 1
-  };
   GstV4l2Src *v4l2src = GST_V4L2SRC (basesrc);
   GstV4l2Object *obj = v4l2src->v4l2object;
   GList *caps_list = NULL;
@@ -403,20 +403,8 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps, GstStructure * pref_s)
   GstV4l2Error error = GST_V4L2_ERROR_INIT;
   GstCaps *fcaps = NULL;
 
-  GST_DEBUG_OBJECT (basesrc, "fixating caps %" GST_PTR_FORMAT, caps);
-
-  /* We consider the first structure from peercaps to be a preference. This is
-   * useful for matching a reported native display, or simply to avoid
-   * transformation to happen downstream. */
-  if (pref_s) {
-    pref_s = gst_structure_copy (pref_s);
-    gst_v4l2_src_fixate_struct_with_preference (pref_s, &pref);
-    gst_v4l2_src_parse_fixed_struct (pref_s, &pref.width, &pref.height,
-        &pref.fps_n, &pref.fps_d);
-    gst_structure_free (pref_s);
-  }
-
-  GST_DEBUG_OBJECT (basesrc, "Preferred size %ix%i", pref.width, pref.height);
+  GST_DEBUG_OBJECT (basesrc, "Fixating caps %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (basesrc, "Preferred size %ix%i", pref->width, pref->height);
 
   /* Sort the structures to get the caps that is nearest to our preferences,
    * first. Use single struct caps for sorting so we preserve the features.  */
@@ -424,10 +412,10 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps, GstStructure * pref_s)
     GstCaps *tmp = gst_caps_copy_nth (caps, i);
 
     s = gst_caps_get_structure (tmp, 0);
-    gst_v4l2_src_fixate_struct_with_preference (s, &pref);
+    gst_v4l2_src_fixate_struct_with_preference (s, pref);
 
     caps_list = g_list_insert_sorted_with_data (caps_list, tmp,
-        (GCompareDataFunc) gst_v4l2src_fixed_caps_compare, &pref);
+        (GCompareDataFunc) gst_v4l2src_fixed_caps_compare, pref);
   }
 
   gst_caps_unref (caps);
@@ -502,13 +490,111 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps, GstStructure * pref_s)
   return fcaps;
 }
 
+static gboolean
+gst_v4l2src_query_preferred_dv_timings (GstV4l2Src * v4l2src,
+    struct PreferredCapsInfo *pref)
+{
+  GstV4l2Object *obj = v4l2src->v4l2object;
+  struct v4l2_dv_timings dv_timings = { 0, };
+  const struct v4l2_bt_timings *bt = &dv_timings.bt;
+  gint tot_width, tot_height;
+  gint gcd;
+
+  if (!gst_v4l2_query_dv_timings (obj, &dv_timings))
+    return FALSE;
+
+  pref->width = bt->width;
+  pref->height = bt->height;
+
+  tot_height = bt->height +
+      bt->vfrontporch + bt->vsync + bt->vbackporch +
+      bt->il_vfrontporch + bt->il_vsync + bt->il_vbackporch;
+  tot_width = bt->width + bt->hfrontporch + bt->hsync + bt->hbackporch;
+
+  pref->fps_n = bt->pixelclock;
+  pref->fps_d = tot_width * tot_height;
+
+  if (bt->interlaced)
+    pref->fps_d /= 2;
+
+  gcd = gst_util_greatest_common_divisor (pref->fps_n, pref->fps_d);
+  pref->fps_n /= gcd;
+  pref->fps_d /= gcd;
+
+  /* If are are not streaming (e.g. we received source-change event), lock the
+   * new timing immediatly so that TRY_FMT can properly work */
+  if (!obj->pool || !GST_V4L2_BUFFER_POOL_IS_STREAMING (obj->pool)) {
+    gst_v4l2_set_dv_timings (obj, &dv_timings);
+    /* Setting a new DV timings invalidates the probed caps. */
+    gst_caps_replace (&obj->probed_caps, NULL);
+  }
+
+  GST_INFO_OBJECT (v4l2src, "Using DV Timings: %i x %i (%i/%i fps)",
+      pref->width, pref->height, pref->fps_n, pref->fps_d);
+
+  return TRUE;
+}
+
+static gboolean
+gst_v4l2src_query_preferred_size (GstV4l2Src * v4l2src,
+    struct PreferredCapsInfo *pref)
+{
+  struct v4l2_input in = { 0, };
+
+  if (!gst_v4l2_get_input (v4l2src->v4l2object, &in.index))
+    return FALSE;
+
+  if (!gst_v4l2_query_input (v4l2src->v4l2object, &in))
+    return FALSE;
+
+  GST_INFO_OBJECT (v4l2src, "Detect input %u as `%s`", in.index, in.name);
+
+  /* Notify signal status using WARNING/INFO messages */
+  if (in.status & (V4L2_IN_ST_NO_POWER | V4L2_IN_ST_NO_SIGNAL)) {
+    if (!v4l2src->no_signal)
+      /* note: taken from decklinksrc element */
+      GST_ELEMENT_WARNING (v4l2src, RESOURCE, READ, ("Signal lost"),
+          ("No input source was detected - video frames invalid"));
+    v4l2src->no_signal = TRUE;
+  } else if (v4l2src->no_signal) {
+    if (v4l2src->no_signal)
+      GST_ELEMENT_INFO (v4l2src, RESOURCE, READ,
+          ("Signal recovered"), ("Input source detected"));
+    v4l2src->no_signal = FALSE;
+  }
+
+  if (in.capabilities & V4L2_IN_CAP_NATIVE_SIZE) {
+    GST_FIXME_OBJECT (v4l2src, "missing support for native video size");
+    return FALSE;
+  } else if (in.capabilities & V4L2_IN_CAP_DV_TIMINGS) {
+    return gst_v4l2src_query_preferred_dv_timings (v4l2src, pref);
+  } else if (in.capabilities & V4L2_IN_CAP_STD) {
+    GST_FIXME_OBJECT (v4l2src, "missing support for video standards");
+    return FALSE;
+  }
+
+  return FALSE;
+}
+
 static gboolean
 gst_v4l2src_negotiate (GstBaseSrc * basesrc)
 {
+  GstV4l2Src *v4l2src = GST_V4L2SRC (basesrc);
   GstCaps *thiscaps;
   GstCaps *caps = NULL;
   GstCaps *peercaps = NULL;
   gboolean result = FALSE;
+  /* Let's prefer a good resolution as of today's standard. */
+  struct PreferredCapsInfo pref = {
+    3840, 2160, 120, 1
+  };
+  gboolean have_pref;
+
+  /* For drivers that has DV timings or other default size query
+   * capabilities, we will prefer that resolution. This must happen before we
+   * probe the caps, as locking DV Timings or standards will change result of
+   * the caps enumeration. */
+  have_pref = gst_v4l2src_query_preferred_size (v4l2src, &pref);
 
   /* first see what is possible on our source pad */
   thiscaps = gst_pad_query_caps (GST_BASE_SRC_PAD (basesrc), NULL);
@@ -537,12 +623,20 @@ gst_v4l2src_negotiate (GstBaseSrc * basesrc)
   if (caps) {
     /* now fixate */
     if (!gst_caps_is_empty (caps)) {
-      GstStructure *pref = NULL;
 
-      if (peercaps && !gst_caps_is_any (peercaps))
-        pref = gst_caps_get_structure (peercaps, 0);
+      /* otherwise consider the first structure from peercaps to be a
+       * preference. This is useful for matching a reported native display,
+       * or simply to avoid transformation to happen downstream. */
+      if (!have_pref && peercaps && !gst_caps_is_any (peercaps)) {
+        GstStructure *pref_s = gst_caps_get_structure (peercaps, 0);
+        pref_s = gst_structure_copy (pref_s);
+        gst_v4l2_src_fixate_struct_with_preference (pref_s, &pref);
+        gst_v4l2_src_parse_fixed_struct (pref_s, &pref.width, &pref.height,
+            &pref.fps_n, &pref.fps_d);
+        gst_structure_free (pref_s);
+      }
 
-      caps = gst_v4l2src_fixate (basesrc, caps, pref);
+      caps = gst_v4l2src_fixate (basesrc, caps, &pref);
 
       /* Fixating may fail as we now set the selected format */
       if (!caps) {
@@ -550,7 +644,7 @@ gst_v4l2src_negotiate (GstBaseSrc * basesrc)
         goto done;
       }
 
-      GST_DEBUG_OBJECT (basesrc, "fixated to: %" GST_PTR_FORMAT, caps);
+      GST_INFO_OBJECT (basesrc, "fixated to: %" GST_PTR_FORMAT, caps);
 
       if (gst_caps_is_any (caps)) {
         /* hmm, still anything, so element can do anything and
@@ -572,7 +666,7 @@ done:
 
 no_nego_needed:
   {
-    GST_DEBUG_OBJECT (basesrc, "no negotiation needed");
+    GST_INFO_OBJECT (basesrc, "no negotiation needed");
     if (thiscaps)
       gst_caps_unref (thiscaps);
     return TRUE;
@@ -606,8 +700,14 @@ gst_v4l2src_decide_allocation (GstBaseSrc * bsrc, GstQuery * query)
     GstV4l2Error error = GST_V4L2_ERROR_INIT;
 
     caps = gst_caps_make_writable (caps);
-    if (!(ret = gst_v4l2src_set_format (src, caps, &error)))
+
+    ret = gst_v4l2src_set_format (src, caps, &error);
+    if (ret) {
+      GstV4l2BufferPool *pool = GST_V4L2_BUFFER_POOL (src->v4l2object->pool);
+      gst_v4l2_buffer_pool_enable_resolution_change (pool);
+    } else {
       gst_v4l2_error (src, &error);
+    }
 
     gst_caps_unref (caps);
     src->pending_set_fmt = FALSE;
@@ -829,7 +929,6 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
 {
   GstV4l2Src *v4l2src = GST_V4L2SRC (src);
   GstV4l2Object *obj = v4l2src->v4l2object;
-  GstV4l2BufferPool *pool = GST_V4L2_BUFFER_POOL_CAST (obj->pool);
   GstFlowReturn ret;
   GstClock *clock;
   GstClockTime abs_time, base_time, timestamp, duration;
@@ -838,15 +937,38 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
   gboolean half_frame;
 
   do {
+    GstV4l2BufferPool *pool = GST_V4L2_BUFFER_POOL_CAST (obj->pool);
+
     ret = GST_BASE_SRC_CLASS (parent_class)->alloc (GST_BASE_SRC (src), 0,
         obj->info.size, buf);
 
-    if (G_UNLIKELY (ret != GST_FLOW_OK))
+    if (G_UNLIKELY (ret != GST_FLOW_OK)) {
+      if (ret == GST_V4L2_FLOW_RESOLUTION_CHANGE) {
+        GST_INFO_OBJECT (v4l2src, "Resolution change detected.");
+
+        /* It is required to always cycle through streamoff, we also need to
+         * streamoff in order to allow locking a new DV_TIMING which will
+         * influence the output of TRY_FMT */
+        gst_v4l2src_stop (GST_BASE_SRC (src));
+
+        /* Force renegotiation */
+        v4l2src->renegotiation_adjust = v4l2src->offset + 1;
+        v4l2src->pending_set_fmt = TRUE;
+
+        if (!gst_base_src_negotiate (GST_BASE_SRC (src))) {
+          ret = GST_FLOW_NOT_NEGOTIATED;
+          goto error;
+        }
+
+        continue;
+      }
       goto alloc_failed;
+    }
 
-    ret = gst_v4l2_buffer_pool_process (pool, buf);
+    ret = gst_v4l2_buffer_pool_process (pool, buf, NULL);
 
-  } while (ret == GST_V4L2_FLOW_CORRUPTED_BUFFER);
+  } while (ret == GST_V4L2_FLOW_CORRUPTED_BUFFER ||
+      ret == GST_V4L2_FLOW_RESOLUTION_CHANGE);
 
   if (G_UNLIKELY (ret != GST_FLOW_OK))
     goto error;
