@@ -23,24 +23,22 @@
 
 #define SIMPLE_WAV_PATH GST_TEST_FILES_PATH G_DIR_SEPARATOR_S "audiotestsrc.wav"
 
-static void
-do_test_simple_file (GstPadMode mode)
+static GstElement *
+create_pipeline (GstPadMode mode)
 {
-  GstStateChangeReturn ret;
   GstElement *pipeline;
   GstElement *src, *q = NULL;
   GstElement *wavparse;
   GstElement *fakesink;
-  GstMessage *msg;
 
   pipeline = gst_pipeline_new ("testpipe");
-  src = gst_element_factory_make ("filesrc", NULL);
+  src = gst_element_factory_make ("filesrc", "filesrc");
   fail_if (src == NULL);
   if (mode == GST_PAD_MODE_PUSH)
-    q = gst_element_factory_make ("queue", NULL);
-  wavparse = gst_element_factory_make ("wavparse", NULL);
+    q = gst_element_factory_make ("queue", "queue");
+  wavparse = gst_element_factory_make ("wavparse", "wavparse");
   fail_if (wavparse == NULL);
-  fakesink = gst_element_factory_make ("fakesink", NULL);
+  fakesink = gst_element_factory_make ("fakesink", "fakesink");
   fail_if (fakesink == NULL);
 
   gst_bin_add_many (GST_BIN (pipeline), src, wavparse, fakesink, q, NULL);
@@ -52,6 +50,18 @@ do_test_simple_file (GstPadMode mode)
   else
     fail_unless (gst_element_link_many (src, wavparse, fakesink, NULL));
 
+  return pipeline;
+}
+
+static void
+do_test_simple_file (GstPadMode mode)
+{
+  GstStateChangeReturn ret;
+  GstElement *pipeline;
+  GstMessage *msg;
+
+  pipeline = create_pipeline (mode);
+
   ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
   fail_unless_equals_int (ret, GST_STATE_CHANGE_ASYNC);
 
@@ -133,6 +143,72 @@ GST_START_TEST (test_empty_file_push)
 
 GST_END_TEST;
 
+static GstPadProbeReturn
+fakesink_buffer_cb (GstPad * sink, GstPadProbeInfo * info, gpointer user_data)
+{
+  GstClockTime *ts = user_data;
+  GstBuffer *buf = GST_PAD_PROBE_INFO_BUFFER (info);
+
+  fail_unless (buf);
+  if (!GST_CLOCK_TIME_IS_VALID (*ts))
+    *ts = GST_BUFFER_PTS (buf);
+
+  return GST_PAD_PROBE_OK;
+}
+
+GST_START_TEST (test_seek)
+{
+  GstStateChangeReturn ret;
+  GstElement *pipeline;
+  GstMessage *msg;
+  GstElement *wavparse, *fakesink;
+  GstPad *pad;
+  GstClockTime seek_position = (20 * GST_MSECOND);
+  GstClockTime first_ts = GST_CLOCK_TIME_NONE;
+
+  pipeline = create_pipeline (GST_PAD_MODE_PULL);
+  wavparse = gst_bin_get_by_name (GST_BIN (pipeline), "wavparse");
+  fail_unless (wavparse);
+  fakesink = gst_bin_get_by_name (GST_BIN (pipeline), "fakesink");
+  fail_unless (fakesink);
+
+  pad = gst_element_get_static_pad (fakesink, "sink");
+  fail_unless (pad);
+  gst_pad_add_probe (pad, GST_PAD_PROBE_TYPE_BUFFER, fakesink_buffer_cb,
+      &first_ts, NULL);
+  gst_object_unref (pad);
+
+  /* wavparse is able to seek in the READY state */
+  ret = gst_element_set_state (pipeline, GST_STATE_READY);
+  fail_unless_equals_int (ret, GST_STATE_CHANGE_SUCCESS);
+
+  fail_unless (gst_element_seek_simple (wavparse, GST_FORMAT_TIME,
+          GST_SEEK_FLAG_FLUSH, seek_position));
+
+  ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  fail_unless_equals_int (ret, GST_STATE_CHANGE_ASYNC);
+
+  ret = gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
+  fail_unless_equals_int (ret, GST_STATE_CHANGE_SUCCESS);
+
+  msg = gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline),
+      GST_CLOCK_TIME_NONE, GST_MESSAGE_EOS | GST_MESSAGE_ERROR);
+
+  /* check that the first buffer produced by wavparse matches the seek
+     position we requested */
+  fail_unless_equals_clocktime (first_ts, seek_position);
+
+  fail_unless_equals_string (GST_MESSAGE_TYPE_NAME (msg), "eos");
+
+  gst_message_unref (msg);
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (wavparse);
+  gst_object_unref (fakesink);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 static Suite *
 wavparse_suite (void)
 {
@@ -144,6 +220,7 @@ wavparse_suite (void)
   tcase_add_test (tc_chain, test_empty_file_push);
   tcase_add_test (tc_chain, test_simple_file_pull);
   tcase_add_test (tc_chain, test_simple_file_push);
+  tcase_add_test (tc_chain, test_seek);
   return s;
 }
 
